/*
 * ConversionHandler.h
 *
 *  Created on: 23. 2. 2014
 *      Author: Tomas Pecka
 */

#ifndef ALGORITHMCHOOSER_H_
#define ALGORITHMCHOOSER_H_

#include <deque>
#include <sax/Token.h>

/**
 * Wraps all conversions.
 * Determine what formalism input, what is desired output and algorithm. Then perform conversion.
 */
class ConversionHandler {
public:
	enum TAlgorithm {
		DEFAULT,
		 /* FA to RE + RG to RE */
		BRZOZOWSKI_ALGEBRAIC, STATE_ELIMINATION,
		 /* FA to RG */
		INCOMING_TRANSITIONS, OUTGOING_TRANSITIONS,
		 /* RE to FA + RE to RG */
		BRZOZOWSKI_DERIVATION, THOMPSON_NFA, GLUSHKOV_NFA,
		/* RG to FA */

		 /* CFG to PDA */
		BOTTOM_UP, TOP_DOWN,
		 /* RTE to PDA */
		GLUSHKOV_RTE_NAIVE, GLUSHKOV_RTE,
	};

	enum TFormalism {
		 // Regular language
		FINITE_AUTOMATON, REGULAR_GRAMMAR, REGULAR_EXPRESSION,

		 // CFG Languages
		PUSHDOWN_AUTOMATON,	CONTEXTFREE_GRAMMAR,

		 // Tree languages
		FINITE_TREE_AUTOMATON, REGULAR_TREE_EXPRESSION,
	};

	ConversionHandler ( ext::deque < sax::Token > && tokens, const std::string & target, const std::string & algorithm );
	void convert ( void );

private:
	TFormalism parseFormalismFromString ( const std::string & _target ) const;
	TFormalism parseFormalismFromTokens ( void ) const;
	TAlgorithm parseAlgorithmFromString ( const std::string & _algorithm ) const;

	void convertFA ( void );
	void convertFAtoRE ( void );
	void convertFAtoRG ( void );

	void convertRE ( void );
	void convertREtoFA ( void );
	void convertREtoRG ( void );

	void convertRG ( void );
	void convertRGtoFA ( void );
	void convertRGtoRE ( void );

	void convertPDA ( void );

	void convertCFG ( void );
	void convertCFGtoPDA ( void );

	void convertRTE ( void );
	void convertRTEtoPDA ( void );

	void convertFTA ( void );
	void convertFTAtoPDA ( void );

	ext::deque < sax::Token > m_tokens;
	TFormalism m_source, m_target;
	TAlgorithm m_algorithm;
};

#endif /* ALGORITHMCHOOSER_H_ */
