/*
 * aconversion.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <sax/FromXMLParserHelper.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/CommonException.h>

#include "ConversionHandler.h"

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Converts between regular expressions, regular grammars and finite automaton.", ' ', "0.01" );

		std::vector < std::string > formalisms {
			"fa", "re", "rg", "pda"
		};
		TCLAP::ValuesConstraint < std::string > allowedFormalisms ( formalisms );
		TCLAP::ValueArg < std::string > target ( "t", "target", "Specifies target formalism", true, "", & allowedFormalisms );
		cmd.add ( target );

		std::vector < std::string > algorithms {
			"algebraic", "elimination", "brzozowski", "glushkov", "thompson", "incoming", "outgoing", "bottomup", "topdown", "default", "glushkovrtenaive", "glushkovrte"
		};
		TCLAP::ValuesConstraint < std::string > allowedAlgorithms ( algorithms );
		TCLAP::ValueArg < std::string > algorithm ( "a", "algorithm", "Specifies algorithm to use", false, "default", & allowedAlgorithms );
		cmd.add ( algorithm );

		TCLAP::ValueArg < std::string > input ( "i", "input", "input to convert", false, "-", "file" );
		cmd.add ( input );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		ext::deque < sax::Token > inputTokens;

		ConversionHandler conv( sax::FromXMLParserHelper::parseInput(input), target.getValue(), algorithm.getValue() );
		conv.convert( );

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) ext::cmeasure << measurements::results ( ) << std::endl;

		return 0;
	} catch(const exception::CommonException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cerr << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
