/*
 * ExactTreePatternAutomaton.cpp
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#include "ExactTreePatternAutomaton.h"
#include <alphabet/SubtreeWildcardSymbol.h>
#include <tree/Tree.h>
#include <tree/ranked/PrefixRankedTree.h>
#include <automaton/Automaton.h>
#include <automaton/PDA/InputDrivenNPDA.h>

#include <deque>
#include <registration/AlgoRegistration.hpp>

namespace arbology {

namespace exact {

automaton::Automaton ExactTreePatternAutomaton::construct ( const tree::Tree & tree, const DefaultSymbolType & subtreeWildcard ) {
	return dispatch ( tree.getData ( ), subtreeWildcard );
}

automaton::InputDrivenNPDA < > ExactTreePatternAutomaton::construct ( const tree::PrefixRankedTree < > & tree, const DefaultSymbolType & subtreeWildcard ) {
	DefaultSymbolType S = DefaultSymbolType ( 'S' );
	automaton::InputDrivenNPDA < > res ( DefaultStateType ( 0 ), S );

	for ( const common::ranked_symbol < > & rankedSymbol : tree.getAlphabet ( ) ) {
		DefaultSymbolType symbol ( alphabet::RankedSymbol < > { rankedSymbol } );
		res.addInputSymbol ( symbol );
		res.setPushdownStoreOperation ( symbol, ext::vector < DefaultSymbolType > ( 1, S ), ext::vector < DefaultSymbolType > ( ( size_t ) rankedSymbol.getRank ( ), S ) );
	}

	res.addInputSymbol ( subtreeWildcard );
	res.setPushdownStoreOperation ( subtreeWildcard, ext::vector < DefaultSymbolType > ( 1, S ), ext::vector < DefaultSymbolType > { } );

	int i = 1;
	ext::deque < std::pair < size_t, int > > subtreeJumps;

	for ( const common::ranked_symbol < > & rankedSymbol : tree.getContent ( ) ) {
		DefaultSymbolType symbol ( alphabet::RankedSymbol < > { rankedSymbol } );
		subtreeJumps.push_back ( std::make_pair ( ( size_t ) rankedSymbol.getRank ( ), i - 1 ) );

		res.addState ( DefaultStateType ( i ) );
		res.addTransition ( DefaultStateType ( i - 1 ), symbol, DefaultStateType ( i ) );
		res.addTransition ( DefaultStateType ( 0 ), std::move ( symbol ), DefaultStateType ( i ) );

		while ( subtreeJumps.size ( ) && subtreeJumps.back ( ).first == 0 ) {
			res.addTransition ( DefaultStateType ( subtreeJumps.back ( ).second ), subtreeWildcard, DefaultStateType ( i ) );
			subtreeJumps.pop_back ( );
		}

		if ( subtreeJumps.size ( ) ) subtreeJumps.back ( ).first--;

		i++;
	}

	return res;
}

auto ExactTreePatternAutomatonPrefixRankedTree = registration::OverloadRegister < ExactTreePatternAutomaton, automaton::InputDrivenNPDA < >, tree::PrefixRankedTree < > > ( ExactTreePatternAutomaton::construct );

} /* namespace exact */

} /* namespace arbology */
