/*
 * NFADeterminizer.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Travnicek
 */

#include "common/RHDPDACommon.h"

#include <automaton/PDA/RealTimeHeightDeterministicNPDA.h>
#include <set>
#include <global/GlobalData.h>

namespace automaton {

namespace determinize {

void addRetTransition(const DefaultStateType& from, const ext::variant<DefaultEpsilonType, DefaultSymbolType>& input, const DefaultSymbolType& dvpdaSymbol, const DefaultStateType& to, automaton::RealTimeHeightDeterministicDPDA < > & deterministic) {
	deterministic.addState(from);
	deterministic.addState(to);
	deterministic.addPushdownStoreSymbol(dvpdaSymbol);

	deterministic.addReturnTransition(from, input, dvpdaSymbol, to);
}

void retInitial(const DefaultStateType& state, const DefaultSymbolType& pdaSymbol, const ext::variant<DefaultEpsilonType, DefaultSymbolType>& input, const automaton::RealTimeHeightDeterministicNPDA < > & nondeterministic, automaton::RealTimeHeightDeterministicDPDA < > & deterministic, ext::map<ext::tuple<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>, DefaultSymbolType>, DefaultStateType>& rubbishReturnTransitions) {
	const ext::set<ext::pair<DefaultStateType, DefaultStateType>> & S = unpackFromStateLabel(state);

	ext::set<ext::pair<DefaultStateType, DefaultStateType>> S1;
	for(const auto& entry : S) {
		const DefaultStateType& q = entry.first;
		const DefaultStateType& q2 = entry.second;

		for(const auto& transition : nondeterministic.getReturnTransitions()) {
			if(q2 != std::get<0>(transition.first)) continue;
			if(input != std::get<1>(transition.first)) continue;
			if(nondeterministic.getBottomOfTheStackSymbol() != std::get<2>(transition.first)) continue;

			for(const auto& to : transition.second) {
				const DefaultStateType& q1 = to;
				S1.insert(ext::make_pair(q, q1));
			}
		}
	}

	bool S1Empty = S1.empty();
	DefaultStateType to(packToStateLabel(std::move(S1)));
	if(S1Empty) {
		ext::tuple<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>, DefaultSymbolType> key(state, input, pdaSymbol);
		rubbishReturnTransitions.insert(std::make_pair(key, to));
	} else {
		addRetTransition(state, input, pdaSymbol, to, deterministic);
	}
}

void ret(const DefaultStateType& state, const DefaultSymbolType& pdaSymbol, const ext::variant<DefaultEpsilonType, DefaultSymbolType>& input, const automaton::RealTimeHeightDeterministicNPDA < > & nondeterministic, automaton::RealTimeHeightDeterministicDPDA < > & deterministic, ext::map<ext::tuple<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>, DefaultSymbolType>, DefaultStateType>& rubbishReturnTransitions) {
	const ext::set<ext::pair<DefaultStateType, DefaultStateType>> & S = unpackFromStateLabel(state);
	const ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>> & pdaSymbolUnpack = unpackFromDRHDPDAStackSymbol(pdaSymbol);
	const ext::set<ext::pair<DefaultStateType, DefaultStateType>>& S1 = unpackFromStateLabel ( pdaSymbolUnpack.first );

	ext::set<ext::pair<DefaultStateType, DefaultStateType>> update;

	for(const auto& transition : nondeterministic.getCallTransitions()) {
		if(pdaSymbolUnpack.second != std::get<1>(transition.first)) continue;

		const DefaultStateType& q = std::get<0>(transition.first);
		for(const auto& to : transition.second) {
			const DefaultStateType& q1 = to.first;
			const DefaultSymbolType& gamma = to.second;

			for(const auto& entry : S) {
				if(q1 != entry.first) continue;
				const DefaultStateType& q2 = entry.second;

				for(const auto& transition2 : nondeterministic.getReturnTransitions()) {
					if(q2 != std::get<0>(transition2.first)) continue;
					if(input != std::get<1>(transition2.first)) continue;
					if(gamma != std::get<2>(transition2.first)) continue;

					for(const auto& to2 : transition2.second) {
						const DefaultStateType& qI = to2;

						update.insert(ext::make_pair(q, qI));
					}
				}
			}
		}
	}

	ext::set<ext::pair<DefaultStateType, DefaultStateType>> S2;
	for(const auto& entry : S1) {
		const DefaultStateType& q = entry.first;
		const DefaultStateType& q3 = entry.second;
		for(const auto& entry2 : update) {
			if(q3 != entry2.first) continue;

			const DefaultStateType& qI = entry2.second;

			S2.insert(ext::make_pair(q, qI));
		}
	}

	bool S2Empty = S2.empty();
	DefaultStateType to(packToStateLabel(std::move(S2)));
	if(S2Empty) {
		ext::tuple<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>, DefaultSymbolType> key(state, input, pdaSymbol);
		rubbishReturnTransitions.insert(std::make_pair(key, to));
	} else {
		addRetTransition(state, input, pdaSymbol, to, deterministic);
	}
}

void addCallTransition(const DefaultStateType& from, const ext::variant<DefaultEpsilonType, DefaultSymbolType>& input, const DefaultStateType& to, const DefaultSymbolType& dvpdaSymbol, automaton::RealTimeHeightDeterministicDPDA < > & deterministic) {
	deterministic.addState(from);
	deterministic.addState(to);
	deterministic.addPushdownStoreSymbol(dvpdaSymbol);

	deterministic.addCallTransition(from, input, to, dvpdaSymbol);
}

void call(const DefaultStateType& state, const ext::variant<DefaultEpsilonType, DefaultSymbolType>& input, const automaton::RealTimeHeightDeterministicNPDA < > & nondeterministic, automaton::RealTimeHeightDeterministicDPDA < > & deterministic, ext::map<ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>>, ext::pair<DefaultStateType, DefaultSymbolType> >& rubbishCallTransitions) {
	const ext::set<ext::pair<DefaultStateType, DefaultStateType>> & S = unpackFromStateLabel(state);

	ext::set<DefaultStateType> R = retrieveDSubSet(S);
	ext::set<DefaultStateType> R1;

	for(const DefaultStateType& q : R) {
		for(const auto& transition : nondeterministic.getCallTransitions()) {
			if(q != transition.first.first) continue;
			if(input != transition.first.second) continue;

			for(const auto& to : transition.second) {
				const DefaultStateType& q1 = to.first;

				R1.insert(q1);
			}
		}
	}

	bool R1Empty = R1.empty();
	DefaultStateType to = packToStateLabel(createIdentity(std::move(R1)));
	DefaultSymbolType push = packToStackSymbolLabel(ext::make_pair(state, input));
	if(R1Empty) {
		rubbishCallTransitions.insert(std::make_pair(ext::make_pair(state, input), ext::make_pair(to, push)));
	} else {
		addCallTransition(state, input, to, push, deterministic);
	}
}

void addLocalTransition(const DefaultStateType& from, const ext::variant<DefaultEpsilonType, DefaultSymbolType>& input, const DefaultStateType& to, automaton::RealTimeHeightDeterministicDPDA < > & deterministic) {
	deterministic.addState(from);
	deterministic.addState(to);

	deterministic.addLocalTransition(from, input, to);
}

void local(const DefaultStateType& state, const ext::variant<DefaultEpsilonType, DefaultSymbolType>& input, const automaton::RealTimeHeightDeterministicNPDA < > & nondeterministic, automaton::RealTimeHeightDeterministicDPDA < > & deterministic, ext::map<ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>>, DefaultStateType>& rubbishLocalTransitions ) {
	const ext::set<ext::pair<DefaultStateType, DefaultStateType>> & S = unpackFromStateLabel(state);
	ext::set<ext::pair<DefaultStateType, DefaultStateType>> S1;

	for(const auto& entry : S) {
		const DefaultStateType & q = entry.first;
		const DefaultStateType & q2 = entry.second;
		for(const auto& transition : nondeterministic.getLocalTransitions()) {
			if(q2 != transition.first.first) continue;
			if(input != transition.first.second) continue;

			for(const auto& to : transition.second) {
				const DefaultStateType & q1 = to;

				S1.insert(ext::make_pair(q, q1));
			}
		}
	}

	bool S1Empty = S1.empty();
	DefaultStateType to(packToStateLabel(std::move(S1)));
	if(S1Empty) {
		rubbishLocalTransitions.insert(std::make_pair(ext::make_pair(state, input), to));
	} else {
		addLocalTransition(state, input, to, deterministic);
	}
}

ext::tuple<ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>, ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>, ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>> getLocalCallRetPartitioning(const automaton::RealTimeHeightDeterministicNPDA < > & n, const DefaultStateType& state) {
	ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>> local;
	ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>> call;
	ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>> ret;

	const ext::set<DefaultStateType> dSubSet = retrieveDSubSet(unpackFromStateLabel(state));

	for(const auto& transition : n.getLocalTransitions()) {
		if(dSubSet.count(transition.first.first))
			local.insert(transition.first.second);
	}

	for(const auto& transition : n.getCallTransitions()) {
		if(dSubSet.count(transition.first.first))
			call.insert(transition.first.second);
	}

	for(const auto& transition : n.getReturnTransitions()) {
		if(dSubSet.count(std::get<0>(transition.first)))
			ret.insert(std::get<1>(transition.first));
	}

	return ext::make_tuple(std::move(local), std::move(call), std::move(ret));
}

automaton::RealTimeHeightDeterministicDPDA < > Determinize::determinize(const automaton::RealTimeHeightDeterministicNPDA < > & n) {
	DefaultStateType initialLabel = packToStateLabel(createIdentity(n.getInitialStates()));

	automaton::RealTimeHeightDeterministicDPDA < > d(initialLabel, n.getBottomOfTheStackSymbol());
	d.setInputAlphabet(n.getInputAlphabet());

	ext::set<DefaultStateType> rubbishStates = {DefaultStateType(packToStateLabel({}))};
	ext::map<ext::tuple<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>, DefaultSymbolType>, DefaultStateType> rubbishReturnTransitions;
	ext::map<ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>>, ext::pair<DefaultStateType, DefaultSymbolType> > rubbishCallTransitions;
	ext::map<ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>>, DefaultStateType> rubbishLocalTransitions;

	for(;;) {
		ext::set<ext::pair<DefaultStateType, DefaultSymbolType>> stateSymbols = existsDirtyStateSymbol(d, rubbishStates, rubbishCallTransitions, rubbishReturnTransitions, n);
		ext::set<DefaultStateType> states = existsDirtyState(d, rubbishStates, rubbishCallTransitions, rubbishLocalTransitions, n);

		if(stateSymbols.empty() && states.empty()) break;

		for(const auto& stateSymbol : stateSymbols) {
			if ( common::GlobalData::verbose )
				std::clog << "Dirty state symbol: " << stateSymbol << std::endl;

			ext::tuple<ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>, ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>, ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>> partitioning = getLocalCallRetPartitioning(n, stateSymbol.first);

			ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>& retPart = std::get<2>(partitioning);

			for(const auto& symbol : retPart) {
				if(stateSymbol.second == d.getBottomOfTheStackSymbol()) {
					retInitial(stateSymbol.first, stateSymbol.second, symbol, n, d, rubbishReturnTransitions);
				} else {
					ret(stateSymbol.first, stateSymbol.second, symbol, n, d, rubbishReturnTransitions);
				}
			}
		}

		for(const auto& state : states) {
			if ( common::GlobalData::verbose )
				std::clog << "Dirty state: " << state << std::endl;
			ext::tuple<ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>, ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>, ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>> partitioning = getLocalCallRetPartitioning(n, state);

			ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>& localPart = std::get<0>(partitioning);
			ext::set<ext::variant<DefaultEpsilonType, DefaultSymbolType>>& callPart = std::get<1>(partitioning);

			for(const auto& symbol : localPart) {
				local(state, symbol, n, d, rubbishLocalTransitions);
			}
			for(const auto& symbol : callPart) {
				call(state, symbol, n, d, rubbishCallTransitions);
			}
		}
	}

	const ext::set<DefaultStateType>& finalLabels = n.getFinalStates();
	for(const DefaultStateType& state : d.getStates()) {
		const ext::set<DefaultStateType> labels = retrieveDSubSet(unpackFromStateLabel(state));

		if(!ext::excludes(finalLabels.begin(), finalLabels.end(), labels.begin(), labels.end())) {
			d.addFinalState(state);
		}
	}

	if ( common::GlobalData::verbose ) {
		std::clog << "Rubbish states: " << rubbishStates << std::endl;
		std::clog << "Rubbish return transitions: " << rubbishReturnTransitions << std::endl;
		std::clog << "Rubbish call transitions: " << rubbishCallTransitions << std::endl;
		std::clog << "Rubbish local transitions:" << rubbishLocalTransitions << std::endl;
	}

	return d;
}

auto DeterminizeRealTimeHeightDeterministicNPDA = registration::OverloadRegister < Determinize, automaton::RealTimeHeightDeterministicDPDA < >, automaton::RealTimeHeightDeterministicNPDA < > > ( Determinize::determinize );
auto DeterminizeRealTimeHeightDeterministicNPDA2 = registration::AbstractRegister < Determinize, automaton::RealTimeHeightDeterministicDPDA < >, const automaton::RealTimeHeightDeterministicNPDA < > & > ( Determinize::determinize );

} /* namespace determinize */

} /* namespace automaton */
