/*
 * ReachableStates.h
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef REACHABLE_STATES_H_
#define REACHABLE_STATES_H_

#include <core/multipleDispatch.hpp>
#include <algorithm>
#include <deque>
#include <set>
#include <map>
#include <queue>

#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>

#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

namespace automaton {

namespace properties {

class ReachableStates : public alib::SingleDispatch<ReachableStates, ext::set<DefaultStateType>, const automaton::AutomatonBase &> {
public:
	static ext::set<DefaultStateType> reachableStates( const automaton::Automaton & automaton );

	/**
	 * Removes dead states from FSM. Melichar 2.29
	 */
	template < class T, class SymbolType = typename automaton::SymbolTypeOfAutomaton < T >, class StateType = typename automaton::StateTypeOfAutomaton < T > >
	static ext::set<StateType> reachableStates( const T & fsm );
	template < class SymbolType, class StateType >
	static ext::set<StateType> reachableStates( const automaton::MultiInitialStateNFA < SymbolType, StateType > & fsm );
	template < class SymbolType, class StateType >
	static ext::set<StateType> reachableStates( const automaton::DFA < SymbolType, StateType > & fsm );
};

template < class T, class SymbolType, class StateType >
ext::set<StateType> ReachableStates::reachableStates( const T & fsm ) {
	// 1a
	ext::deque<ext::set<StateType>> Qi;
	Qi.push_back( ext::set<StateType>( ) );
	Qi.at( 0 ) = {fsm.getInitialState( )};

	int i = 1;

	// 1bc
	while( true ) {
		Qi.push_back( Qi.at( i - 1 ) );

		for( const auto & p : Qi.at( i - 1 ) )
			for( const auto & transition : fsm.getTransitionsFromState( p ) )
				Qi.at( i ).insert( transition.second.begin(), transition.second.end() );

		if( Qi.at( i ) == Qi.at( i - 1 ) )
			break;

		i = i + 1;
	}

	return Qi.at( i );
}

template < class SymbolType, class StateType >
ext::set<StateType> ReachableStates::reachableStates( const automaton::MultiInitialStateNFA < SymbolType, StateType > & fsm ) {
	// 1a
	ext::deque<ext::set<StateType>> Qi;
	Qi.push_back( ext::set<StateType>( ) );
	Qi.at( 0 ) = fsm.getInitialStates( );

	int i = 1;

	// 1bc
	while( true ) {
		Qi.push_back( Qi.at( i - 1 ) );

		for( const auto & p : Qi.at( i - 1 ) )
			for( const auto & transition : fsm.getTransitionsFromState( p ) )
				Qi.at( i ).insert( transition.second.begin(), transition.second.end() );

		if( Qi.at( i ) == Qi.at( i - 1 ) )
			break;

		i = i + 1;
	}

	return Qi.at( i );
}

template < class SymbolType, class StateType >
ext::set<StateType> ReachableStates::reachableStates( const automaton::DFA < SymbolType, StateType > & fsm ) {
	// 1a
	ext::deque<ext::set<StateType>> Qi;
	Qi.push_back( ext::set<StateType>( ) );
	Qi.at( 0 ). insert( fsm.getInitialState( ) );

	int i = 1;

	// 1bc
	while( true ) {
		Qi.push_back( Qi.at( i - 1 ) );

		for( const auto & p : Qi.at( i - 1 ) )
			for( const auto & transition : fsm.getTransitionsFromState( p ) )
				Qi.at( i ).insert( transition.second );

		if( Qi.at( i ) == Qi.at( i - 1 ) )
			break;

		i = i + 1;
	}

	return Qi.at( i );
}

} /* namespace properties */

} /* namespace automaton */

#endif /* REACHABLE_STATES_H_ */
