/*
 * UsefullStates.h
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef USEFULL_STATES_H_
#define USEFULL_STATES_H_

#include <core/multipleDispatch.hpp>
#include <algorithm>
#include <deque>
#include <set>
#include <map>

#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>

#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

namespace automaton {

namespace properties {

class UsefullStates : public alib::SingleDispatch<UsefullStates, ext::set<DefaultStateType>, const automaton::AutomatonBase &> {
public:
	static ext::set<DefaultStateType> usefullStates( const automaton::Automaton & automaton );

	/**
	 * Removes dead states from FSM. Melichar 2.32
	 */
	template < class T, class SymbolType = typename automaton::SymbolTypeOfAutomaton < T >, class StateType = typename automaton::StateTypeOfAutomaton < T > >
	static ext::set<StateType> usefullStates( const T & fsm );
	template < class SymbolType, class StateType >
	static ext::set<StateType> usefullStates( const automaton::DFA < SymbolType, StateType > & fsm );
};

template < class T, class SymbolType, class StateType >
ext::set<StateType> UsefullStates::usefullStates( const T & fsm ) {
	// 1a
	ext::deque<ext::set<StateType>> Qi;
	Qi.push_back( ext::set<StateType>( ) );
	Qi.at( 0 ) = fsm.getFinalStates( );

	int i = 1;

	// 1bc
	while( true ) {
		Qi.push_back( Qi.at( i - 1 ) ); // copy Qi-1 into Qi
		for( const auto & p : Qi.at( i - 1 ) )
			for( const auto & t : fsm.getTransitionsToState( p ) )
				Qi.at( i ).insert( t.first.first );

		if( Qi.at( i ) == Qi.at( i - 1 ) )
			break;

		i = i + 1;
	}
	return Qi.at( i );
}

template < class SymbolType, class StateType >
ext::set<StateType> UsefullStates::usefullStates( const automaton::DFA < SymbolType, StateType > & fsm ) {
	// 1a
	ext::deque<ext::set<StateType>> Qi;
	Qi.push_back( ext::set<StateType>( ) );
	Qi.at( 0 ) = fsm.getFinalStates( );

	int i = 1;

	// 1bc
	while( true ) {
		Qi.push_back( Qi.at( i - 1 ) ); // copy Qi-1 into Qi
		for( const auto & p : Qi.at( i - 1 ) )
			for( const auto & t : fsm.getTransitionsToState( p ) )
				Qi.at( i ).insert( t.first.first );

		if( Qi.at( i ) == Qi.at( i - 1 ) )
			break;

		i = i + 1;
	}
	return Qi.at( i );
}

} /* namespace properties */

} /* namespace automaton */

#endif /* USEFULL_STATES_H_ */
