/*
 * EpsilonRemoverOutgoing.h
 *
 *  Created on: 16. 1. 2014
 *	  Author: Tomas Pecka
 */

#ifndef EPSILON_REMOVER_OUTGOING_H_
#define EPSILON_REMOVER_OUTGOING_H_

#include <core/multipleDispatch.hpp>
#include <automaton/Automaton.h>

#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include "../properties/EpsilonClosure.h"

namespace automaton {

namespace simplify {

class EpsilonRemoverOutgoing : public alib::SingleDispatch<EpsilonRemoverOutgoing, automaton::Automaton, const automaton::AutomatonBase &> {
public:
	static automaton::Automaton remove( const automaton::Automaton & automaton );

	/**
	 * Computes epsilon closure of a state in epsilon nonfree automaton
	 */
	template < class StateType, class EpsilonType, class SymbolType >
	static automaton::MultiInitialStateNFA < SymbolType, StateType > remove( const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & fsm );
	template < class StateType, class SymbolType >
	static automaton::MultiInitialStateNFA < SymbolType, StateType > remove( const automaton::MultiInitialStateNFA < SymbolType, StateType > & fsm );
	template < class StateType, class SymbolType >
	static automaton::NFA < SymbolType, StateType > remove( const automaton::NFA < SymbolType, StateType > & fsm );
	template < class StateType, class SymbolType >
	static automaton::DFA < SymbolType, StateType > remove( const automaton::DFA < SymbolType, StateType > & fsm );
};

template < class StateType, class SymbolType >
automaton::DFA < SymbolType, StateType > EpsilonRemoverOutgoing::remove(const automaton::DFA < SymbolType, StateType > & origFSM) {
	return origFSM;
}

template < class StateType, class SymbolType >
automaton::MultiInitialStateNFA < SymbolType, StateType > EpsilonRemoverOutgoing::remove(const automaton::MultiInitialStateNFA < SymbolType, StateType > & origFSM) {
	return origFSM;
}

template < class StateType, class SymbolType >
automaton::NFA < SymbolType, StateType > EpsilonRemoverOutgoing::remove(const automaton::NFA < SymbolType, StateType > & origFSM) {
	return origFSM;
}

template < class StateType, class EpsilonType, class SymbolType >
automaton::MultiInitialStateNFA < SymbolType, StateType > EpsilonRemoverOutgoing::remove( const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & origFSM ) {
	automaton::MultiInitialStateNFA < SymbolType, StateType > fsm;
	fsm.setStates( origFSM.getStates() );
	fsm.setFinalStates( origFSM.getFinalStates() );
	fsm.setInputAlphabet( origFSM.getInputAlphabet() );

	/**
	 * Step 1 from Melichar 2.41
	 */
	for( const auto & middle : origFSM.getStates( ) ) {
		const ext::set<StateType> middleClosure = automaton::properties::EpsilonClosure::epsilonClosure( origFSM, middle );
		for( const auto & symbol : origFSM.getInputAlphabet() ) {
			for( const auto& transition : origFSM.getTransitions( ) ) {
				if(transition.second.find(middle) == transition.second.end()) continue;
				if(transition.first.second != symbol) continue;

				for( const auto & to : middleClosure )
					fsm.addTransition(transition.first.first, symbol, to );
			}
		}
	}

	/**
	 * Step 2 from Melichar 2.41
	 */
	const ext::set<StateType> cl = automaton::properties::EpsilonClosure::epsilonClosure( origFSM, origFSM.getInitialState() );
	fsm.setInitialStates( cl );

	return fsm;
}

} /* namespace simplify */

} /* namespace automaton */

#endif /* EPSILON_REMOVER_OUTGOING_H_ */
