/*
 * GenerateUpToLength.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Jan Travnicek
 */

#ifndef GRAMMAR_GENERATE_UP_TO_LENGTH_H_
#define GRAMMAR_GENERATE_UP_TO_LENGTH_H_

#include <core/multipleDispatch.hpp>

#include <grammar/Grammar.h>
#include <string/LinearString.h>
#include <set>
#include <deque>

#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightRG.h>

namespace grammar {

namespace generate {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class GenerateUpToLength : public alib::SingleDispatch < GenerateUpToLength, ext::set < string::LinearString < > >, const grammar::GrammarBase &, unsigned > {
public:
	static ext::set < string::LinearString < > > generate( const grammar::Grammar & grammar, unsigned length );

	template < class T, class SymbolType = typename grammar::SymbolTypeOfGrammar < T > >
	static ext::set < string::LinearString < SymbolType > > generate ( const T & grammar, unsigned length );
};

template < class T, class SymbolType >
ext::set < string::LinearString < SymbolType > > GenerateUpToLength::generate ( const T & grammar, unsigned length ) {
	ext::set < string::LinearString < SymbolType > > res;

	ext::map < SymbolType, ext::set < ext::vector < SymbolType > > > rules = grammar.getRawRules();
	if ( grammar.getGeneratesEpsilon ( ) ) {
		res.insert ( string::LinearString < SymbolType > { } );
		rules [ grammar.getInitialSymbol ( ) ].erase ( ext::vector < SymbolType > { } );
	}

	ext::deque < std::pair < ext::vector < SymbolType >, ext::vector < SymbolType > > > data;
	data.push_back ( std::make_pair ( ext::vector < SymbolType > { }, ext::vector < SymbolType > { grammar.getInitialSymbol ( ) } ) );

	while ( ! data.empty ( ) ) {
		std::pair < ext::vector < SymbolType >, ext::vector < SymbolType > > item = std::move ( data.back ( ) );
		data.pop_back ( );

		SymbolType nonterminal = std::move ( item.second.back ( ) );
		item.second.pop_back ( );
		if ( rules.find ( nonterminal ) == rules.end ( ) ) continue;

		for ( const ext::vector < SymbolType > & rhs : rules.find ( nonterminal )->second ) {
			if ( item.first.size ( ) + item.second.size ( ) + rhs.size ( ) > length ) continue;

			ext::vector < SymbolType > newStack ( item.second );
			ext::vector < SymbolType > newString ( item.first );
			newStack.insert ( newStack.end ( ), rhs.rbegin ( ), rhs.rend ( ) );

			while ( ! newStack.empty ( ) && grammar.getTerminalAlphabet ( ).count ( newStack.back ( ) ) ) {
				newString.push_back ( std::move(newStack.back ( ) ) );
				newStack.pop_back ( );
			}

			if ( newStack.empty ( ) ) {
				res.insert ( string::LinearString < SymbolType > ( newString ) );
			} else {
				data.push_back ( std::make_pair ( std::move ( newString ), std::move ( newStack ) ) );
			}
		}
	}

	return res;
}

} /* namespace generate */

} /* namespace grammar */

#endif /* GRAMMAR_GENERATE_UP_TO_LENGTH_H_ */
