/*
 * GrammarPropertiesCFG.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef PRODUCTIVE_NONTERMINALS_H_
#define PRODUCTIVE_NONTERMINALS_H_

#include <core/multipleDispatch.hpp>

#include <grammar/Grammar.h>

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include <set>
#include <deque>
#include <algorithm>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class ProductiveNonterminals : public alib::SingleDispatch<ProductiveNonterminals, ext::set<DefaultSymbolType>, const grammar::GrammarBase &> {
public:
	static ext::set<DefaultSymbolType> getProductiveNonterminals( const grammar::Grammar & grammar );

	/**
	 * Implements steps 1 through 3 in Melichar 3.6
	 */
	template<class T, class SymbolType = typename grammar::SymbolTypeOfGrammar < T > >
	static ext::set<SymbolType> getProductiveNonterminals( const T & grammar );
};

template<class T, class SymbolType >
ext::set<SymbolType> ProductiveNonterminals::getProductiveNonterminals( const T & grammar ) {
	// 1.
	ext::deque<ext::set<SymbolType>> Ni;
	Ni.push_back( ext::set<SymbolType>( ) );

	int i = 1;

	// 2.
	while( true ) {
		Ni.push_back( Ni.at( i - 1 ) );

		for( const auto & rule : grammar.getRawRules( ) ) {
			for( const auto & rhs : rule.second ) {
				if( std::all_of( rhs.begin( ), rhs.end( ), [ i, Ni, grammar ]( const SymbolType & symbol ) -> bool {
					return Ni.at( i - 1 ) . count( symbol ) || grammar.getTerminalAlphabet( ). count( symbol );
				} ) )
					Ni.at( i ).insert( rule.first );
			}
		}

		if( Ni.at( i ) == Ni.at( i - 1 ) )
			break;

		i = i + 1;
	}

	// 3.
	return Ni.at( i );
}

} /* namespace properties */

} /* namespace grammar */

#endif /* PRODUCTIVE_NONTERMINALS_H_ */
