/*
 * RegExpToAutomaton.h
 *
 *  Created on: 11. 1. 2014
 *	  Author: Tomas Pecka
 */

#ifndef REG_EXP_TO_AUTOMATON_H_
#define REG_EXP_TO_AUTOMATON_H_

#include <core/multipleDispatch.hpp>

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

#include <automaton/Automaton.h>

#include "ToAutomatonGlushkov.h"

namespace regexp {

namespace convert {

class ToAutomaton : public alib::SingleDispatch<ToAutomaton, automaton::Automaton, const regexp::RegExpBase &> {
public:
	/**
	 * Performs conversion.
	 * @return FSM equivalent to original regular expression.
	 */
	static automaton::Automaton convert(const regexp::RegExp& regexp);

	template < class SymbolType >
	static automaton::Automaton convert(const regexp::FormalRegExp < SymbolType > & regexp);
	template < class SymbolType >
	static automaton::Automaton convert(const regexp::UnboundedRegExp < SymbolType > & regexp);

};

template < class SymbolType >
automaton::Automaton ToAutomaton::convert(const regexp::FormalRegExp < SymbolType > & regexp) {
	return automaton::Automaton(ToAutomatonGlushkov::convert(regexp));
}

template < class SymbolType >
automaton::Automaton ToAutomaton::convert(const regexp::UnboundedRegExp < SymbolType > & regexp) {
	return automaton::Automaton(ToAutomatonGlushkov::convert(regexp));
}

} /* namespace convert */

} /* namespace regexp */

#endif /* REG_EXP_TO_AUTOMATON_H_ */
