/*
 * Author: Radovan Cerveny
 */

#ifndef EXACT_SUFFIX_AUTOMATON_H_
#define EXACT_SUFFIX_AUTOMATON_H_

#include <automaton/Automaton.h>
#include <automaton/FSM/DFA.h>
#include <string/LinearString.h>
#include <string/String.h>
#include <core/multipleDispatch.hpp>

namespace stringology {

namespace indexing {

class ExactSuffixAutomaton : public alib::SingleDispatch < ExactSuffixAutomaton, automaton::Automaton, const string::StringBase & > {
private:
	static void suffixAutomatonAddSymbol ( automaton::DFA < > & suffixAutomaton, const DefaultSymbolType & symbol, ext::map < DefaultStateType, std::pair<DefaultStateType, int > > & suffixLinks, DefaultStateType & lastState );

public:
	 /**
	 * Linear time on-line construction of minimal suffix automaton for given pattern.
	 * @return minimal suffix automaton for given pattern.
	 */
	static automaton::Automaton construct ( const string::String & pattern );

	static automaton::DFA < > construct ( const string::LinearString < > & pattern );

};

} /* namespace indexing */

} /* namespace stringology */

#endif /* EXACT_SUFFIX_AUTOMATON_H_ */
