/*
 * TopologicalSort.cpp
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#include "TopologicalSort.h"
#include "../traverse/Dfs.h"
#include <registration/AlgoRegistration.hpp>

namespace graph {

namespace sort {

static TopologicalSort::Result topsort_impl(const DirectedGraph &graph) {
	TopologicalSort::Result s;
	ext::unordered_map<Node, bool> visited;

	for (const Node &n : graph.getNodes()) {
		traverse::Dfs::dfs(graph, n, [&s, &visited](const Node &node, const Node&, int, int) {
			if (visited.find(node) != visited.end()) {
				return false;
			}
			visited.insert({node, true});
			s.push_front(node);
			return true;
		});
	}

	return s;
}

TopologicalSort::Result TopologicalSort::topologicalsort(const Graph &graph) {
	return dispatch(graph.getData());
}

TopologicalSort::Result TopologicalSort::topologicalsort(const DirectedGraph &graph) {
	return topsort_impl(graph);
}

auto TopologicalSortDirectedGraph = registration::OverloadRegister < TopologicalSort, TopologicalSort::Result, graph::DirectedGraph> ( TopologicalSort::topologicalsort );

} // namespace sort

} // namespace graph
