/*
 * StringAccess.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "StringAccess.h"

#include <string/LinearString.h>
#include <string/CyclicString.h>

#include <factory/XmlDataFactory.hpp>
#include <container/ObjectsSet.h>
#include <container/ObjectsMap.h>
#include <container/ObjectsTuple.h>
#include <container/ObjectsVector.h>
#include <container/ObjectsVariant.h>
#include <container/ObjectsPair.h>
#include <container/ObjectsTree.h>

#include "common/AccessHelpers.hpp"
#include <registration/AlgoRegistration.hpp>

namespace dataAccess {

void StringAccess::access ( string::String & string, const StringSettings::Settings & settings, const OperationSettings::Settings & operation, ext::deque < sax::Token > & argument ) {
	dispatch ( string.getData ( ), settings, operation, argument );
}

void StringAccess::access ( string::LinearString < > & string, const StringSettings::Settings & settings, const OperationSettings::Settings & operation, ext::deque < sax::Token > & argument ) {
	if ( settings == StringSettings::Settings::ALPHABET )
		return handleComponent < string::GeneralAlphabet > ( string, operation, argument );

	if ( settings == StringSettings::Settings::CONTENT && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( string.getContent ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto StringAccessFormalString = registration::OverloadRegister < StringAccess, void, string::LinearString < > > ( StringAccess::access );

void StringAccess::access ( string::CyclicString < > & string, const StringSettings::Settings & settings, const OperationSettings::Settings & operation, ext::deque < sax::Token > & argument ) {
	if ( settings == StringSettings::Settings::ALPHABET )
		return handleComponent < string::GeneralAlphabet > ( string, operation, argument );

	if ( settings == StringSettings::Settings::CONTENT && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( string.getContent ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto StringAccessUnboundedString = registration::OverloadRegister < StringAccess, void, string::CyclicString < > > ( StringAccess::access );

} /* namespace dataAccess */
