/*
 * TreeAccess.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "TreeAccess.h"

#include <tree/ranked/RankedTree.h>
#include <tree/ranked/RankedPattern.h>
#include <tree/ranked/RankedNonlinearPattern.h>
#include <tree/unranked/UnrankedTree.h>

#include <factory/XmlDataFactory.hpp>
#include <container/ObjectsSet.h>
#include <container/ObjectsMap.h>
#include <container/ObjectsTuple.h>
#include <container/ObjectsVector.h>
#include <container/ObjectsVariant.h>
#include <container/ObjectsPair.h>
#include <container/ObjectsTree.h>

#include "common/AccessHelpers.hpp"
#include <registration/AlgoRegistration.hpp>

namespace dataAccess {

void TreeAccess::access ( tree::Tree & tree, const TreeSettings::Settings & settings, const OperationSettings::Settings & operation, ext::deque < sax::Token > & argument ) {
	dispatch ( tree.getData ( ), settings, operation, argument );
}

void TreeAccess::access ( tree::RankedTree < > & tree, const TreeSettings::Settings & settings, const OperationSettings::Settings & operation, ext::deque < sax::Token > & argument ) {
	if ( settings == TreeSettings::Settings::ALPHABET )
		return handleComponent < tree::GeneralAlphabet > ( tree, operation, argument );

	if ( settings == TreeSettings::Settings::CONTENT && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( tree.getContent ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto TreeAccessRankedTree = registration::OverloadRegister < TreeAccess, void, tree::RankedTree < > > ( TreeAccess::access );

void TreeAccess::access ( tree::RankedPattern < > & tree, const TreeSettings::Settings & settings, const OperationSettings::Settings & operation, ext::deque < sax::Token > & argument ) {
	if ( settings == TreeSettings::Settings::ALPHABET )
		return handleComponent < tree::GeneralAlphabet > ( tree, operation, argument );

	if ( settings == TreeSettings::Settings::CONTENT && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( tree.getContent ( ) );

	if ( settings == TreeSettings::Settings::SUBTREE_WILDCARD && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( tree.getSubtreeWildcard ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto TreeAccessRankedPattern = registration::OverloadRegister < TreeAccess, void, tree::RankedPattern < > > ( TreeAccess::access );

void TreeAccess::access ( tree::RankedNonlinearPattern < > & tree, const TreeSettings::Settings & settings, const OperationSettings::Settings & operation, ext::deque < sax::Token > & argument ) {
	if ( settings == TreeSettings::Settings::ALPHABET )
		return handleComponent < tree::GeneralAlphabet > ( tree, operation, argument );

	if ( settings == TreeSettings::Settings::CONTENT )
		return alib::XmlDataFactory::toStdout ( tree.getContent ( ) );

	if ( settings == TreeSettings::Settings::SUBTREE_WILDCARD && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( tree.getSubtreeWildcard ( ) );

	if ( settings == TreeSettings::Settings::NONLINEAR_VARIABLES && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( tree.getNonlinearVariables ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto TreeAccessRankedNonlinearPattern = registration::OverloadRegister < TreeAccess, void, tree::RankedNonlinearPattern < > > ( TreeAccess::access );

void TreeAccess::access ( tree::UnrankedTree < > & tree, const TreeSettings::Settings & settings, const OperationSettings::Settings & operation, ext::deque < sax::Token > & argument ) {
	if ( settings == TreeSettings::Settings::ALPHABET )
		return handleComponent < tree::GeneralAlphabet > ( tree, operation, argument );

	if ( settings == TreeSettings::Settings::CONTENT && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( tree.getContent ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto TreeAccessUnrankedTree = registration::OverloadRegister < TreeAccess, void, tree::UnrankedTree < > > ( TreeAccess::access );

} /* namespace dataAccess */
