/*
 * SettingsHelper.h
 *
 *  Created on: 26. 3. 2014
 *	  Author: Jan Travnicek
 */

#ifndef __SETTINGS_HELPER_H__
#define __SETTINGS_HELPER_H__

#include <string>
#include <map>
#include <vector>
#include <exception/CommonException.h>

namespace dataAccess {

template < class EnumClass >
class EnumClassHelper {
	static const std::vector < typename EnumClass::Settings > & vectorInit ( ) {
		static std::vector < typename EnumClass::Settings > res;

		for ( int i = 0; i < static_cast < int > ( EnumClass::Settings::__MAX__ ); i++ )
			res.push_back ( static_cast < typename EnumClass::Settings > ( i ) );

		return res;
	}

public:
	static const std::vector < typename EnumClass::Settings > & settingsVector ( ) {
		static std::vector < typename EnumClass::Settings > res = vectorInit ( );

		return res;
	}

private:
	static const std::map < std::string, typename EnumClass::Settings > & mapInit ( ) {
		static std::map < std::string, typename EnumClass::Settings > res;

		for ( typename EnumClass::Settings setting : settingsVector ( ) )
			res.insert ( std::make_pair ( EnumClass::toString ( setting ), setting ) );

		return res;
	}

	static const std::vector < std::string > & stringsInit ( ) {
		static std::vector < std::string > res;

		for ( typename EnumClass::Settings setting : settingsVector ( ) )
			res.push_back ( EnumClass::toString ( setting ) );

		return res;
	}

public:
	static const std::map < std::string, typename EnumClass::Settings > & settingsMap ( ) {
		static std::map < std::string, typename EnumClass::Settings > res = mapInit ( );

		return res;
	}

	static const std::vector < std::string > & stringsVector ( ) {
		static const std::vector < std::string > & res = stringsInit ( );

		return res;
	}

	static typename EnumClass::Settings fromString ( const std::string & str ) {
		const std::map < std::string, typename EnumClass::Settings > & mapping = settingsMap ( );

		typename std::map < std::string, typename EnumClass::Settings >::const_iterator iter = mapping.find ( str );

		if ( iter == mapping.end ( ) )
			throw exception::CommonException ( "Enumeration element not found" );

		return iter->second;
	}

};

} /* namespace dataAccess */

#endif /* __SETTINGS_HELPER_H__ */
