/*
 * StringDiff.h
 *
 *  Created on: Apr 4, 2017
 *      Author: Tomas Pecka
 */

#ifndef STRING_DIFF_H_
#define STRING_DIFF_H_

#include <core/multipleDispatch.hpp>

#include <compare/DiffAux.h>
#include <compare/StringCompare.h>

#include <string/String.h>
#include <string/StringFeatures.h>
#include <utility>
#include <vector>
#include <ostream>

#include "string/CyclicString.h"
#include "string/LinearString.h"

namespace compare {

class StringDiff : public alib::MultipleDispatch < StringDiff, int, ext::tuple < >, ext::tuple < const string::StringBase &, const string::StringBase & >, ext::tuple < std::ostream & > > {
private:
	template < class SymbolType >
	static void printDiff ( const string::LinearString < SymbolType > & a, const string::LinearString < SymbolType > & b, std::ostream & out );

	template < class SymbolType >
	static void printDiff ( const string::CyclicString < SymbolType > & a, const string::CyclicString < SymbolType > & b, std::ostream & out );

public:
	template < class T >
	static int diff ( const T & a, const T & b, std::ostream & out );

	static int diff ( const string::String & a, const string::String & b, std::ostream & out );
};

template < class SymbolType >
void StringDiff::printDiff ( const string::CyclicString < SymbolType > &, const string::CyclicString < SymbolType > &, std::ostream & ) {
	throw "NYI";
}

template < class SymbolType >
void StringDiff::printDiff ( const string::LinearString < SymbolType > & a, const string::LinearString < SymbolType > & b, std::ostream & out ) {
	out << "StringsComparer" << std::endl;

	if ( a.getAlphabet ( ) != b.getAlphabet ( ) ) {
		out << "Alphabet" << std::endl;

		DiffAux::setDiff ( a.getAlphabet ( ), b.getAlphabet ( ) );
	}

	if ( a.getContent ( ) != b.getContent ( ) ) {
		out << "Content" << std::endl;

		DiffAux::vectorDiff ( a.getContent ( ), b.getContent ( ) );
	}
}

template < class T >
int StringDiff::diff ( const T & a, const T & b, std::ostream & out ) {
	if ( !StringCompare::compare ( a, b ) ) {
		StringDiff::printDiff ( a, b, out );
		return 1;
	} else {
		return 0;
	}
}

} /* namespace compare */

#endif /* STRING_DIFF_H_ */
