/*
 * Registry.cpp
 *
 *  Created on: 10. 7. 2017
 *	  Author: Jan Travnicek
 */

#include <abstraction/Registry.h>

#include <abstraction/AlgorithmRegistry.hpp>
#include <abstraction/ImmediateRegistry.hpp>
#include <abstraction/XmlParserRegistry.hpp>
#include <abstraction/XmlFileWriterRegistry.hpp>
#include <abstraction/ValuePrinterRegistry.hpp>
#include <abstraction/CastRegistry.hpp>
#include <abstraction/NormalizeRegistry.hpp>
#include <abstraction/DowncastRegistry.hpp>

#include <core/namingApi.hpp>

namespace abstraction {

ext::set < std::string > Registry::listAlgorithmGroup ( const std::string & group ) {
	return AlgorithmRegistry::listGroup ( group );
}

ext::set < std::string > Registry::listAlgorithms ( ) {
	return AlgorithmRegistry::list ( );
}

ext::set < std::string > Registry::listDataTypes ( ) {
	return XmlFileWriterRegistry::list ( );
}

ext::set < std::string > Registry::listDataTypeGroup ( const std::string & group ) {
	return XmlFileWriterRegistry::listGroup ( group );
}

ext::set < std::string > Registry::listCastsFrom ( const std::string & type ) {
	return CastRegistry::listFrom ( type );
}

ext::set < std::string > Registry::listCastsTo ( const std::string & type ) {
	return CastRegistry::listTo ( type );
}

ext::set < ext::pair < std::string, std::string > > Registry::listCasts ( ) {
	return CastRegistry::list ( );
}

ext::set < ext::pair < std::string, ext::set < std::string > > > Registry::listNames ( ) {
	return alib::namingApi::list ( );
}

ext::set < ext::pair < ext::pair < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier > >, ext::vector < ext::tuple < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier >, std::string > > > > Registry::listOverloads ( const std::string & algorithm ) {
	return AlgorithmRegistry::listOverloads ( algorithm );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getAlgorithmAbstraction ( const std::string & name, const ext::vector < std::string > & paramTypes ) {
	return AlgorithmRegistry::getAbstraction ( name, paramTypes );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getAlgorithmAbstraction ( const std::string & name, const ext::vector < std::string > & paramTypes, bool & unwrap, bool & normalize ) {
	return AlgorithmRegistry::getAbstraction ( name, paramTypes, unwrap, normalize );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getCastAbstraction ( const std::string & target, const std::string & param, bool & normalize ) {
	return CastRegistry::getAbstraction ( target, param, normalize );
}

bool Registry::isCastNoOp ( const std::string & target, const std::string & param ) {
	return CastRegistry::isNoOp ( target, param );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getImmediateAbstraction ( const std::string & result, std::string value ) {
	return ImmediateRegistry::getAbstraction ( result, std::move ( value ) );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getNormalizeAbstraction ( const std::string & param ) {
	return NormalizeRegistry::getAbstraction ( param );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getDowncastAbstraction ( const std::string & concrete, const std::string & base ) {
	return DowncastRegistry::getAbstraction ( concrete, base );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getValuePrinterAbstraction ( const std::string & param ) {
	return ValuePrinterRegistry::getAbstraction ( param );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getXmlFileWriterAbstraction ( const std::string & param, std::string filename ) {
	return XmlFileWriterRegistry::getAbstraction ( param, std::move ( filename ) );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getXmlParserAbstraction ( const std::string & typeName, ext::deque < sax::Token > tokens ) {
	return XmlParserRegistry::getAbstraction ( typeName, std::move ( tokens ) );
}

} /* namespace abstraction */
