/*
 * Registry.h
 *
 *  Created on: 10. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _REGISTRY_H_
#define _REGISTRY_H_

#include <abstraction/OperationAbstraction.hpp>
#include <abstraction/common/ParamQualifiers.hpp>

namespace abstraction {

class Registry {
public:
	static ext::set < std::string > listAlgorithmGroup ( const std::string & group );
	static ext::set < std::string > listAlgorithms ( );

	static ext::set < std::string > listDataTypes ( );
	static ext::set < std::string > listDataTypeGroup ( const std::string & group );

	static ext::set < std::string > listCastsFrom ( const std::string & type );
	static ext::set < std::string > listCastsTo ( const std::string & type );
	static ext::set < ext::pair < std::string, std::string > > listCasts ( );

	static ext::set < ext::pair < ext::pair < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier > >, ext::vector < ext::tuple < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier >, std::string > > > > listOverloads ( const std::string & algorithm );

	static ext::set < ext::pair < std::string, ext::set < std::string > > > listNames ( );

	static std::shared_ptr < abstraction::OperationAbstraction > getAlgorithmAbstraction ( const std::string & name, const ext::vector < std::string > & paramTypes );
	static std::shared_ptr < abstraction::OperationAbstraction > getAlgorithmAbstraction ( const std::string & name, const ext::vector < std::string > & paramTypes, bool & unwrap, bool & normalize );
	static std::shared_ptr < abstraction::OperationAbstraction > getCastAbstraction ( const std::string & target, const std::string & param, bool & normalize );
	static bool isCastNoOp ( const std::string & target, const std::string & param );
	static std::shared_ptr < abstraction::OperationAbstraction > getImmediateAbstraction ( const std::string & result, std::string value );
	static std::shared_ptr < abstraction::OperationAbstraction > getNormalizeAbstraction ( const std::string & param );
	static std::shared_ptr < abstraction::OperationAbstraction > getDowncastAbstraction ( const std::string & concrete, const std::string & base );
	static std::shared_ptr < abstraction::OperationAbstraction > getValuePrinterAbstraction ( const std::string & param );
	static std::shared_ptr < abstraction::OperationAbstraction > getXmlFileWriterAbstraction ( const std::string & param, std::string filename );
	static std::shared_ptr < abstraction::OperationAbstraction > getXmlParserAbstraction ( const std::string & typeName, ext::deque < sax::Token > tokens );
};

} /* namespace abstraction */

#endif /* _REGISTRY_H_ */

