#ifndef _PARAM_QUALIFIERS_HPP_
#define _PARAM_QUALIFIERS_HPP_

#include <set>
#include <type_traits>
#include <core/xmlApi.hpp>
#include <string>

namespace abstraction {

class ParamQualifiers {
public:
	enum class ParamQualifier {
		CONST,
		LREF,
		RREF,
	};

	template < class Param >
	static ext::set < ParamQualifier > paramQualifiers ( ) {
		ext::set < ParamQualifier > res;

		if ( std::is_lvalue_reference < Param >::value )
			res.insert ( ParamQualifier::LREF );

		if ( std::is_rvalue_reference < Param >::value )
			res.insert ( ParamQualifier::RREF );

		if ( std::is_const < typename std::remove_reference < Param >::type >::value )
			res.insert ( ParamQualifier::CONST );

		return res;
	}
};

} /* namespace abstraction */

namespace alib {

template < >
struct xmlApi < abstraction::ParamQualifiers::ParamQualifier > {
	static abstraction::ParamQualifiers::ParamQualifier parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static std::string xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, abstraction::ParamQualifiers::ParamQualifier );
};

} /* namespace alib */

#endif // _PARAM_QUALIFIERS_HPP_
