/*
 * Tuple.h
 *
 * Created on: May 4, 2017
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_BITSET_H_
#define OBJECTS_BITSET_H_

#include <bitset>
#include <string>

#include <core/xmlApi.hpp>
#include <exception/CommonException.h>

#include <primitive/Bool.h>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occurred.
 */
class ObjectsBitset {
private:
	explicit ObjectsBitset ( );

public:
	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "Bitset";

		return xmlTagName;
	}

	template < size_t N >
	static ext::bitset < N > parseRaw ( ext::deque < sax::Token >::iterator & input );

	template < size_t N >
	static void compose ( ext::deque < sax::Token > & out, const ext::bitset < N > & input );
};



template < size_t N >
ext::bitset < N > ObjectsBitset::parseRaw ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, ObjectsBitset::getXmlTagName() );

	ext::bitset < N > res;

	for ( size_t i = 0; i < N; ++i )
		res [ i ] = alib::xmlApi < bool >::parse ( input );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, ObjectsBitset::getXmlTagName() );

	return res;
}

template < size_t N >
void ObjectsBitset::compose ( ext::deque < sax::Token > & out, const ext::bitset < N > & container ) {
	out.emplace_back ( ObjectsBitset::getXmlTagName(), sax::Token::TokenType::START_ELEMENT );

	for ( size_t i = 0; i < N; ++i )
		alib::xmlApi < bool >::compose ( out, container [ i ] );

	out.emplace_back ( ObjectsBitset::getXmlTagName(), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace container */

namespace alib {

template < size_t N >
struct xmlApi < ext::bitset < N > > {
	static ext::bitset < N > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static std::string xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const ext::bitset < N > & data );
};

template < size_t N >
ext::bitset < N > xmlApi < ext::bitset < N > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return container::ObjectsBitset::parseRaw < N > ( input );
}

template < size_t N >
bool xmlApi < ext::bitset < N > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName() );
}

template < size_t N >
std::string xmlApi < ext::bitset < N > >::xmlTagName ( ) {
	return container::ObjectsBitset::getXmlTagName();
}

template < size_t N >
void xmlApi < ext::bitset < N > >::compose ( ext::deque < sax::Token > & output, const ext::bitset < N > & input ) {
	return container::ObjectsBitset::compose < N > ( output, input );
}

} /* namespace alib */

#endif /* OBJECTS_BITSET_H_ */
