/*
 * Tuple.h
 *
 * Created on: May 6, 2016
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_TUPLE_H_
#define OBJECTS_TUPLE_H_

#include <tuple>
#include <string>

#include <core/xmlApi.hpp>
#include <exception/CommonException.h>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occurred.
 */
class ObjectsTuple {
private:
	explicit ObjectsTuple ( );

public:
	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "Tuple";

		return xmlTagName;
	}

	template < class ... Ts >
	static ext::tuple < Ts ... > parseRaw ( ext::deque < sax::Token >::iterator & input );

	template < class ... Ts >
	static void compose ( ext::deque < sax::Token > & out, const ext::tuple < Ts ... > & input );
};



template < typename ... Ts >
ext::tuple < Ts ... > ObjectsTuple::parseRaw ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, ObjectsTuple::getXmlTagName() );

	ext::tuple < Ts ... > res { alib::xmlApi < Ts >::parse ( input ) ... }; // NOTE buggy in gcc < 4.9.1

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, ObjectsTuple::getXmlTagName() );

	return res;
}

template < class ... Ts, size_t ... I >
void tupleComposeHelper ( ext::deque < sax::Token > & out, const ext::tuple < Ts ... > & container, std::index_sequence < I ... > ) {
	( void ) std::initializer_list < int > { ( alib::xmlApi < Ts >::compose ( out, std::get < I > ( container ) ), 0 ) ... }; // FEATURE unpack of multiple packs together
}

template < typename ... Ts >
void ObjectsTuple::compose ( ext::deque < sax::Token > & out, const ext::tuple < Ts ... > & container ) {
	out.emplace_back ( ObjectsTuple::getXmlTagName(), sax::Token::TokenType::START_ELEMENT );

	tupleComposeHelper ( out, container, std::make_index_sequence < ext::tuple_size < ext::tuple < Ts ... > >::value > { } );

	out.emplace_back ( ObjectsTuple::getXmlTagName(), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace container */

namespace alib {

template < typename ... Ts >
struct xmlApi < ext::tuple < Ts ... > > {
	static ext::tuple < Ts ... > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static std::string xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const ext::tuple < Ts ... > & data );
};

template < typename ... Ts >
ext::tuple < Ts ... > xmlApi < ext::tuple < Ts ... > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return container::ObjectsTuple::parseRaw < Ts ... > ( input );
}

template < typename ... Ts >
bool xmlApi < ext::tuple < Ts ... > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName() );
}

template < typename ... Ts >
std::string xmlApi < ext::tuple < Ts ... > >::xmlTagName ( ) {
	return container::ObjectsTuple::getXmlTagName();
}

template < typename ... Ts >
void xmlApi < ext::tuple < Ts ... > >::compose ( ext::deque < sax::Token > & output, const ext::tuple < Ts ... > & input ) {
	return container::ObjectsTuple::compose < Ts ... > ( output, input );
}

} /* namespace alib */

#endif /* OBJECTS_TUPLE_H_ */
