/*
 * namingApi.hpp
 *
 * Created on: Jul 7, 2017
 * Author: Jan Travnicek
 */

#ifndef NAMING_API_HPP_
#define NAMING_API_HPP_

#include <map>
#include <set>
#include <string>
#include <typeindex>

#include <exception/CommonException.h>

namespace alib {

class namingApi {
	static ext::map < std::string, ext::set < ext::type_index > > & getNames ( ) {
		static ext::map < std::string, ext::set < ext::type_index > > names;
		return names;
	}

public:
	template < typename Type >
	static void registerName ( std::string name ) {
		ext::type_index type = ext::type_index ( typeid ( Type ) );
		std::pair < ext::set < ext::type_index >::iterator, bool > res = getNames ( ) [ name ].insert ( type );
		if ( ! res.second && * res.first != type )
			throw exception::CommonException ( "Type for name " + name + " already registered.");
	}

	static ext::set < ext::type_index > & getTypes ( std::string stringname ) {
		auto name = getNames ( ).find ( stringname );
		if ( name == getNames ( ).end ( ) )
			throw exception::CommonException ( "Types for name " + stringname + " not registered.");

		return name->second;
	}

	static bool hasTypes ( std::string stringname ) {
		auto name = getNames ( ).find ( stringname );
		return name != getNames ( ).end ( );
	}

	static ext::set < ext::pair < std::string, ext::set < std::string > > > list ( ) {
		ext::set < ext::pair < std::string, ext::set < std::string > > > res;

		for ( const std::pair < const std::string, ext::set < ext::type_index > > & entry : getNames ( ) ) {
			ext::set < std::string > names;
			for ( const ext::type_index & name : entry.second )
				names.insert ( ext::to_string ( name ) );

			res.insert ( ext::make_pair ( entry.first, names ) );
		}

		return res;
	}

};

} /* namespace alib */

#endif /* CAST_API_HPP_ */
