/*
 * Algorithms.hpp
 *
 *  Created on: 12. 2. 2017
 *      Author: Jan Travnicek
 */

#ifndef _ALGORITHMS_HPP_
#define _ALGORITHMS_HPP_

#include <string>
#include <set>
#include <map>
#include <tuple>
#include <vector>
#include <typeinfo>

namespace introspection {

class Algorithms {
	static ext::map < std::string, ext::tuple < std::string, ext::vector < std::string >, ext::vector < std::string >, ext::vector < std::string > > > & algorithmInterfaces ( ) {
		static ext::map < std::string, ext::tuple < std::string, ext::vector < std::string >, ext::vector < std::string >, ext::vector < std::string > > > res;
		return res;
	}

	static ext::map < std::string, ext::set < ext::tuple < std::string, ext::vector < std::string > > > > & algorithmCallbacks ( ) {
		static ext::map < std::string, ext::set < ext::tuple < std::string, ext::vector < std::string > > > > res;
		return res;
	}

	static ext::set < std::string > & algorithms ( ) {
		static ext::set < std::string > res;
		return res;
	}

	template < class ... Types >
	class TypesToString;

	template < class ... Types >
	class TypesToString < ext::tuple < Types ... > > {
	public:
		static const ext::vector < std::string > & getTypes ( ) {
			static ext::vector < std::string > res { ext::to_string < Types > ( ) ... };

			return res;
		}
	};

public:
	static const ext::map < std::string, ext::tuple < std::string, ext::vector < std::string >, ext::vector < std::string >, ext::vector < std::string > > > & getAlgorithmInterfaces ( ) {
		return algorithmInterfaces ( );
	}

	static const ext::map < std::string, ext::set < ext::tuple < std::string, ext::vector < std::string > > > > & getAlgorithmCallbacks ( ) {
		return algorithmCallbacks ( );
	}

	static const ext::set < std::string > & getAlgorithms ( ) {
		return algorithms ( );
	}

	template < class Algorithm, class RealReturnType, class RealParameterTypes >
	static void registerDispatchInAlgorithm ( ) {
		ext::vector < std::string > realParameterTypeNames = TypesToString < RealParameterTypes >::getTypes ( );
		std::string algorithm = ext::to_string < Algorithm > ( );
		std::string realReturnType = ext::to_string < RealReturnType > ( );

		Algorithms::algorithmCallbacks ( ) [ std::move ( algorithm ) ].insert ( ext::make_tuple ( std::move ( realReturnType ), std::move ( realParameterTypeNames ) ) );
	}

	template < class Algorithm, class ReturnType, class FrontStaticParamTypes, class DispatchedParameterTypes, class BackStaticParamTypes >
	static void registerAlgorithmInterface ( ) {
		std::string algorithm = ext::to_string < Algorithm > ( );
		std::string returnType = ext::to_string < ReturnType > ( );
		ext::vector < std::string > frontStaticParamTypeNames = TypesToString < FrontStaticParamTypes >::getTypes ( );
		ext::vector < std::string > dispatchedParameterTypeNames = TypesToString < DispatchedParameterTypes >::getTypes ( );
		ext::vector < std::string > backStaticParamTypeNames = TypesToString < BackStaticParamTypes >::getTypes ( );

		Algorithms::algorithmInterfaces ( ) [ std::move ( algorithm ) ] = ext::make_tuple ( std::move ( returnType ), std::move ( frontStaticParamTypeNames ), std::move ( dispatchedParameterTypeNames ), std::move ( backStaticParamTypeNames ) );
	}

	template < class Algorithm >
	static void registerAlgorithm ( ) {
		std::string algorithm = ext::to_string < Algorithm > ( );

		Algorithms::algorithms ( ).insert ( std::move ( algorithm ) );
	}
};

} /* namespace introspection */

#endif /* _ALGORITHMS_HPP_ */
