/*
 * AnyObject.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef ANY_OBJECT_H_
#define ANY_OBJECT_H_

#include "ObjectBase.h"
#include <deque>
#include "../sax/Token.h"

#include "../sax/FromXMLParserHelper.h"
#include "Object.h"
#include "../core/xmlApi.hpp"
#include "UniqueObject.h"

namespace alib {

/**
 * Represents void.
 */
template < class T >
class AnyObject final : public ObjectBase {
	T m_data;
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit AnyObject ( const T & data );
	explicit AnyObject ( T && data );

	virtual ObjectBase * clone ( ) const;
	virtual ObjectBase * plunder ( ) &&;

	const T & getData ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const AnyObject & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	void compose ( ext::deque < sax::Token > & out ) const;

	virtual ObjectBase * inc ( ) &&;

	virtual ObjectBase * normalize ( ) && {
		return this;
	}
};

template < class T >
AnyObject < T >::AnyObject ( const T & data ) : m_data ( data ) {
}

template < class T >
AnyObject < T >::AnyObject ( T && data ) : m_data ( std::move ( data ) ) {
}

template < class T >
ObjectBase* AnyObject < T >::clone() const {
	return new AnyObject(*this);
}

template < class T >
ObjectBase* AnyObject < T >::plunder() && {
	return new AnyObject(std::move(*this));
}

template < class T >
const T & AnyObject < T >::getData ( ) const {
	return m_data;
}

template < class T >
int AnyObject < T >::compare(const AnyObject < T > & other ) const {
	static ext::compare < decltype ( m_data ) > comp;

	return comp ( m_data, other.m_data );
}

template < class T >
void AnyObject < T >::operator>>(std::ostream& out) const {
	out << "(AnyObject" << m_data << ")";
}

template < class T >
AnyObject < T > ::operator std::string () const {
	return "V";
}

template < class T >
void AnyObject < T >::compose(ext::deque<sax::Token>& out) const {
	alib::xmlApi < T >::compose(out, m_data);
}

template < class T >
ObjectBase* AnyObject < T >::inc() && {
	return new UniqueObject(Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace alib */

#endif /* ANY_OBJECT_H_ */
