/*
 * Character.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "Character.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/TypeRegistration.hpp>

namespace primitive {

Character::Character(char data) : m_data(data) {

}

PrimitiveBase* Character::clone() const {
	return new Character(*this);
}

PrimitiveBase* Character::plunder() && {
	return new Character(std::move(*this));
}

char Character::getData() const {
	return m_data;
}

Character::operator char ( ) const {
	return m_data;
}

void Character::setData(char data) {
	this->m_data = data;
}

int Character::compare(const Character& other) const {
	return m_data - other.m_data;
}

void Character::operator>>(std::ostream& out) const {
	out << "(Character " << m_data << ")";
}

Character::operator std::string() const {
	return std::string(1, m_data);
}

Character Character::parse(ext::deque<sax::Token>::iterator& input) {
	return Character(parseRaw(input));
}

char Character::parseRaw(ext::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, Character::getXmlTagName());
	char data = sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER)[0];
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, Character::getXmlTagName());
	return data;
}

void Character::compose(ext::deque<sax::Token>& out) const {
	compose(out, this->getData());
}

void Character::compose(ext::deque<sax::Token>& out, char primitive) {
	out.emplace_back(Character::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(std::string(1, primitive), sax::Token::TokenType::CHARACTER);
	out.emplace_back(Character::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

PrimitiveBase * Character::inc() && {
	m_data++;
	return NULL;
}

} /* namespace primitive */

namespace alib {

char xmlApi<char>::parse(ext::deque<sax::Token>::iterator& input) {
	return primitive::Character::parseRaw(input);
}

bool xmlApi<char>::first(const ext::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, xmlTagName());
}

std::string xmlApi<char>::xmlTagName() {
	return primitive::Character::getXmlTagName();
}

void xmlApi<char>::compose(ext::deque<sax::Token>& output, char data) {
	primitive::Character::compose(output, data);
}

auto characterType = registration::TypeRegister < primitive::Primitive, primitive::Character > ();
auto characterType2 = registration::TypeRegister < alib::Object, primitive::Character > ();

} /* namespace alib */

