/*
 * Integer.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_INTEGER_H_
#define PRIMITIVE_INTEGER_H_

#include "PrimitiveBase.h"
#include <core/xmlApi.hpp>

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Integer final : public PrimitiveBase {
protected:
	int m_data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Integer ( int data );

	virtual PrimitiveBase * clone ( ) const;

	virtual PrimitiveBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	int getData ( ) const;

	explicit operator int ( ) const;

	/**
	 * sets new content of primitive
	 */
	void setData ( int data );

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const Integer & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "Integer";

		return xmlTagName;
	}

	static Integer parse ( ext::deque < sax::Token >::iterator & input );
	static int parseRaw ( ext::deque < sax::Token >::iterator & input );

	void compose ( ext::deque < sax::Token > & out ) const;
	static void compose ( ext::deque < sax::Token > & out, int primitive );

	virtual PrimitiveBase * inc ( ) &&;

	typedef Integer normalized_type;

	virtual ObjectBase * normalize ( ) && {
		return this;
	}
};

} /* namespace primitive */

namespace alib {

template < >
struct xmlApi < int > {
	static int parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static std::string xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, int data );
};

} /* namespace alib */

#endif /* PRIMITIVE_INTEGER_H_ */
