/*
 * StringLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_STRING_H_
#define PRIMITIVE_STRING_H_

#include <string>
#include "PrimitiveBase.h"
#include <core/xmlApi.hpp>

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class String final : public PrimitiveBase {
protected:
	std::string m_data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit String ( std::string data );

	explicit String ( const char * data );

	virtual PrimitiveBase * clone ( ) const;

	virtual PrimitiveBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const std::string & getData ( ) const;

	/**
	 * @return name of the symbol
	 */
	std::string & getData ( );

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const String & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "String";

		return xmlTagName;
	}

	static String parse ( ext::deque < sax::Token >::iterator & input );
	static std::string parseRaw ( ext::deque < sax::Token >::iterator & input );

	void compose ( ext::deque < sax::Token > & out ) const;
	static void compose ( ext::deque < sax::Token > & out, const std::string & primitive );

	virtual PrimitiveBase * inc ( ) &&;

	typedef String normalized_type;

	virtual ObjectBase * normalize ( ) && {
		return this;
	}
};

} /* namespace primitive */

namespace alib {

template < >
struct xmlApi < std::string > {
	static std::string parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static std::string xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const std::string & data );
};

} /* namespace alib */

#endif /* PRIMITIVE_STRING_H_ */
