#ifndef _ALGO_REGISTRATION_HPP_
#define _ALGO_REGISTRATION_HPP_

#include <introspection/Algorithms.hpp>
#include <core/multipleDispatch.hpp>

#include <abstraction/AlgorithmRegistry.hpp>

#include <registration/NormalizationRegistration.hpp>

namespace registration {

template < class Algorithm, class RealReturnType, class ... RealParameterTypeBases >
class OverloadRegister {
public:
	OverloadRegister ( typename Algorithm::template rawOverload < RealReturnType, RealParameterTypeBases ... > callback ) {
		Algorithm::template registerOverload < RealReturnType, RealParameterTypeBases ... > ( callback );

		introspection::Algorithms::registerDispatchInAlgorithm < Algorithm, RealReturnType, ext::tuple < RealParameterTypeBases ... > > ( );
		introspection::Algorithms::registerAlgorithmInterface < Algorithm, typename Algorithm::returnType, typename Algorithm::frontParams, typename Algorithm::dispatchedParams, typename Algorithm::backParams > ( );
		introspection::Algorithms::registerAlgorithm < Algorithm > ( );
	}
};

template < class Type, typename enable = void >
struct RequireDowncast : std::false_type {
};

template < class Type >
struct RequireDowncast < Type *, typename std::enable_if < ! std::is_final < Type >::value >::type > : public std::true_type {
};

template < class Algorithm, class ReturnType, class ... ParameterTypes >
class AbstractRegister {
public:
	template < class ... ParamNames, typename std::enable_if < sizeof ... ( ParamNames ) <= sizeof ... ( ParameterTypes ) >::type * = nullptr >
	AbstractRegister ( ReturnType ( * callback ) ( ParameterTypes ... ), ParamNames ... paramNames ) {
		bool normalize = registration::NormalizationRegister < ReturnType > ( ).requireNormalization ( );
		bool downcast = RequireDowncast < ReturnType >::value;

		std::array < std::string, sizeof ... ( ParameterTypes ) > parameterNames = { { paramNames ... } };
		for ( unsigned i = sizeof ... ( ParamNames ); i < sizeof ... ( ParameterTypes ); ++i ) {
			parameterNames [ i ] = "arg" + ext::to_string ( i );
		}

		abstraction::AlgorithmRegistry::registerAlgorithm < Algorithm, ReturnType, ParameterTypes ... > ( callback, downcast, normalize, std::move ( parameterNames ) );
	}
};

} /* namespace registration */

#endif // _ALGO_REGISTRATION_HPP_
