/*
 * InitialSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "InitialSymbol.h"
#include <sax/FromXMLParserHelper.h>
#include "Symbol.h"
#include <object/Object.h>
#include <registration/TypeRegistration.hpp>
#include "UniqueSymbol.h"

namespace alphabet {

InitialSymbol::InitialSymbol() {

}

SymbolBase* InitialSymbol::clone() const {
	return new InitialSymbol(*this);
}

SymbolBase* InitialSymbol::plunder() && {
	return new InitialSymbol(std::move(*this));
}

int InitialSymbol::compare(const InitialSymbol&) const {
	return 0;
}

void InitialSymbol::operator>>(std::ostream& out) const {
	out << "(InitialSymbol)";
}

InitialSymbol::operator std::string() const {
	return InitialSymbol::instance < std::string > ( );
}

InitialSymbol InitialSymbol::parse(ext::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, InitialSymbol::getXmlTagName());
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, InitialSymbol::getXmlTagName());
	return InitialSymbol();
}

void InitialSymbol::compose(ext::deque<sax::Token>& out) const {
	out.emplace_back(InitialSymbol::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(InitialSymbol::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* InitialSymbol::inc() && {
	return new UniqueSymbol(Symbol(std::move(*this)), primitive::Integer(0));
}

} /* namespace alphabet */

namespace alib {

auto initialStateType = registration::TypeRegister < alphabet::Symbol, alphabet::InitialSymbol > ();
auto initialStateType2 = registration::TypeRegister < alib::Object, alphabet::InitialSymbol > ();

} /* namespace alib */
