/*
 * StartSymbol.cpp
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#include "StartSymbol.h"
#include <sax/FromXMLParserHelper.h>
#include "Symbol.h"
#include <object/Object.h>
#include <registration/TypeRegistration.hpp>
#include "UniqueSymbol.h"

namespace alphabet {

StartSymbol::StartSymbol() {

}

SymbolBase* StartSymbol::clone() const {
	return new StartSymbol(*this);
}

SymbolBase* StartSymbol::plunder() && {
	return new StartSymbol(std::move(*this));
}

int StartSymbol::compare(const StartSymbol&) const {
	return 0;
}

void StartSymbol::operator>>(std::ostream& out) const {
	out << "(StartSymbol)";
}

StartSymbol::operator std::string () const {
	return StartSymbol::instance < std::string > ( );
}

StartSymbol StartSymbol::parse(ext::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, StartSymbol::getXmlTagName());
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, StartSymbol::getXmlTagName());
	return StartSymbol();
}

void StartSymbol::compose(ext::deque<sax::Token>& out) const {
	out.emplace_back(StartSymbol::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(StartSymbol::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* StartSymbol::inc() && {
	return new UniqueSymbol(Symbol(std::move(*this)), primitive::Integer(0));
}

} /* namespace alphabet */

namespace alib {

auto startSymbolType = registration::TypeRegister < alphabet::Symbol, alphabet::StartSymbol > ();
auto startSymbolType2 = registration::TypeRegister < alib::Object, alphabet::StartSymbol > ();

} /* namespace alib */
