/*
 * RegExpFromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef REG_EXP_FROM_XML_PARSER_H_
#define REG_EXP_FROM_XML_PARSER_H_

#include <set>
#include <deque>
#include "../RegExpFeatures.h"
#include <sax/Token.h>

#include <sax/FromXMLParserHelper.h>

#include "../unbounded/UnboundedRegExpElements.h"
#include "../formal/FormalRegExpElements.h"
#include <core/xmlApi.hpp>

namespace regexp {

/**
 * Parser used to get RegExp from XML parsed into list of tokens.
 */
class RegExpFromXMLParser {
public:
	template < class SymbolType >
	static ext::set < SymbolType > parseAlphabet ( ext::deque < sax::Token >::iterator & input );

	template < class SymbolType >
	static ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > parseUnboundedRegExpElement ( ext::deque < sax::Token >::iterator & input );

	template < class SymbolType >
	static ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > parseUnboundedRegExpEpsilon ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType >
	static ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > parseUnboundedRegExpEmpty ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType >
	static ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > parseUnboundedRegExpIteration ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType >
	static ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > parseUnboundedRegExpAlternation ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType >
	static ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > parseUnboundedRegExpConcatenation ( ext::deque < sax::Token >::iterator & input );

	template < class SymbolType >
	static ext::rvalue_ref < FormalRegExpElement < SymbolType > > parseFormalRegExpElement ( ext::deque < sax::Token >::iterator & input );

	template < class SymbolType >
	static ext::rvalue_ref < FormalRegExpElement < SymbolType > > parseFormalRegExpEpsilon ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType >
	static ext::rvalue_ref < FormalRegExpElement < SymbolType > > parseFormalRegExpEmpty ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType >
	static ext::rvalue_ref < FormalRegExpElement < SymbolType > > parseFormalRegExpIteration ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType >
	static ext::rvalue_ref < FormalRegExpElement < SymbolType > > parseFormalRegExpAlternation ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType >
	static ext::rvalue_ref < FormalRegExpElement < SymbolType > > parseFormalRegExpConcatenation ( ext::deque < sax::Token >::iterator & input );
};

template < class SymbolType >
ext::set < SymbolType > RegExpFromXMLParser::parseAlphabet ( ext::deque < sax::Token >::iterator & input ) {
	ext::set < SymbolType > alphabet;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alphabet.insert ( alib::xmlApi < SymbolType >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alphabet" );
	return alphabet;
}

template < class SymbolType >
ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > RegExpFromXMLParser::parseUnboundedRegExpElement ( ext::deque < sax::Token >::iterator & input ) {
	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" ) )
		return parseUnboundedRegExpEmpty < SymbolType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "epsilon" ) )
		return parseUnboundedRegExpEpsilon < SymbolType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" ) )
		return parseUnboundedRegExpIteration < SymbolType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" ) )
		return parseUnboundedRegExpAlternation < SymbolType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "concatenation" ) )
		return parseUnboundedRegExpConcatenation < SymbolType > ( input );
	else
		return ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > ( new UnboundedRegExpSymbol < SymbolType > ( alib::xmlApi < SymbolType >::parse ( input ) ) );
}

template < class SymbolType >
ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > RegExpFromXMLParser::parseUnboundedRegExpAlternation ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" );

	UnboundedRegExpAlternation < SymbolType > * alternation = new UnboundedRegExpAlternation < SymbolType > ( );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alternation->appendElement ( parseUnboundedRegExpElement < SymbolType > ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alternation" );
	return ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > ( alternation );
}

template < class SymbolType >
ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > RegExpFromXMLParser::parseUnboundedRegExpConcatenation ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "concatenation" );

	UnboundedRegExpConcatenation < SymbolType > * concatenation = new UnboundedRegExpConcatenation < SymbolType > ( );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		concatenation->appendElement ( parseUnboundedRegExpElement < SymbolType > ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "concatenation" );
	return ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > ( concatenation );
}

template < class SymbolType >
ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > RegExpFromXMLParser::parseUnboundedRegExpIteration ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" );

	ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > iteration ( new UnboundedRegExpIteration < SymbolType > ( parseUnboundedRegExpElement < SymbolType > ( input ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "iteration" );

	return iteration;
}

template < class SymbolType >
ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > RegExpFromXMLParser::parseUnboundedRegExpEpsilon ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "epsilon" );

	ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > epsilon ( new UnboundedRegExpEpsilon < SymbolType > ( ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "epsilon" );

	return epsilon;
}

template < class SymbolType >
ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > RegExpFromXMLParser::parseUnboundedRegExpEmpty ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" );

	ext::rvalue_ref < UnboundedRegExpElement < SymbolType > > empty ( new UnboundedRegExpEmpty < SymbolType > ( ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "empty" );

	return empty;
}

template < class SymbolType >
ext::rvalue_ref < FormalRegExpElement < SymbolType > > RegExpFromXMLParser::parseFormalRegExpElement ( ext::deque < sax::Token >::iterator & input ) {
	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" ) )
		return parseFormalRegExpEmpty < SymbolType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "epsilon" ) )
		return parseFormalRegExpEpsilon < SymbolType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" ) )
		return parseFormalRegExpIteration < SymbolType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" ) )
		return parseFormalRegExpAlternation < SymbolType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "concatenation" ) )
		return parseFormalRegExpConcatenation < SymbolType > ( input );
	else
		return ext::rvalue_ref < FormalRegExpElement < SymbolType > > ( new FormalRegExpSymbol < SymbolType > ( alib::xmlApi < SymbolType >::parse ( input ) ) );
}

template < class SymbolType >
ext::rvalue_ref < FormalRegExpElement < SymbolType > > RegExpFromXMLParser::parseFormalRegExpAlternation ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" );

	ext::rvalue_ref < FormalRegExpElement < SymbolType > > element1 = parseFormalRegExpElement < SymbolType > ( input );
	ext::rvalue_ref < FormalRegExpElement < SymbolType > > element2 = parseFormalRegExpElement < SymbolType > ( input );

	ext::rvalue_ref < FormalRegExpElement < SymbolType > > alternation ( new FormalRegExpAlternation < SymbolType > ( std::move ( element1 ), std::move ( element2 ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alternation" );

	return alternation;
}

template < class SymbolType >
ext::rvalue_ref < FormalRegExpElement < SymbolType > > RegExpFromXMLParser::parseFormalRegExpConcatenation ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "concatenation" );

	ext::rvalue_ref < FormalRegExpElement < SymbolType > > element1 = parseFormalRegExpElement < SymbolType > ( input );
	ext::rvalue_ref < FormalRegExpElement < SymbolType > > element2 = parseFormalRegExpElement < SymbolType > ( input );

	ext::rvalue_ref < FormalRegExpElement < SymbolType > > concatenation ( new FormalRegExpConcatenation < SymbolType > ( std::move ( element1 ), std::move ( element2 ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "concatenation" );

	return concatenation;
}

template < class SymbolType >
ext::rvalue_ref < FormalRegExpElement < SymbolType > > RegExpFromXMLParser::parseFormalRegExpIteration ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" );

	ext::rvalue_ref < FormalRegExpElement < SymbolType > > iteration ( new FormalRegExpIteration < SymbolType > ( parseFormalRegExpElement < SymbolType > ( input ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "iteration" );

	return iteration;
}

template < class SymbolType >
ext::rvalue_ref < FormalRegExpElement < SymbolType > > RegExpFromXMLParser::parseFormalRegExpEpsilon ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "epsilon" );

	ext::rvalue_ref < FormalRegExpElement < SymbolType > > epsilon ( new FormalRegExpEpsilon < SymbolType > ( ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "epsilon" );

	return epsilon;
}

template < class SymbolType >
ext::rvalue_ref < FormalRegExpElement < SymbolType > > RegExpFromXMLParser::parseFormalRegExpEmpty ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" );

	ext::rvalue_ref < FormalRegExpElement < SymbolType > > empty ( new FormalRegExpEmpty < SymbolType > ( ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "empty" );

	return empty;
}

} /* namespace regexp */

#endif /* REG_EXP_FROM_XML_PARSER_H_ */
