/*
 * GraphToXMLComposer.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#ifndef GRAPH_TO_XML_COMPOSER_H_
#define GRAPH_TO_XML_COMPOSER_H_

#include <set>
#include <deque>

#include "../GraphClasses.h"
#include <sax/Token.h>

namespace graph {

// This class contains methods to print XML representation of graph to the output stream.
class GraphToXMLComposer
{
public:
	static void composeNode(ext::deque<sax::Token>& out, const Node &node);
	static void composeEdge(ext::deque<sax::Token>& out, const DirectedEdge &edge);
	static void composeEdge(ext::deque<sax::Token>& out, const UndirectedEdge &edge);

	static void composeRepresentation(ext::deque<sax::Token> &out, const std::string &representation);
	static void composeNodes(ext::deque<sax::Token> &out, const ext::set<Node> &nodes);

	template<typename T>
	static void composeEdges(ext::deque<sax::Token> &out, const ext::set<T> &edges);

	template<typename T>
	static void composeNodeValues(ext::deque<sax::Token> &out, const T &graph);
	template<typename T>
	static void composeEdgeValues(ext::deque<sax::Token> &out, const T &graph);
};

template<typename T>
void GraphToXMLComposer::composeEdges(ext::deque<sax::Token> &out, const ext::set<T> &edges) {
	out.emplace_back("edges", sax::Token::TokenType::START_ELEMENT);
	for (const auto &edge : edges) {
		composeEdge(out, edge);
	}
	out.emplace_back("edges", sax::Token::TokenType::END_ELEMENT);
}

template<typename T>
void GraphToXMLComposer::composeNodeValues(ext::deque<sax::Token> &out, const T &graph) {
	out.emplace_back("nodevalues", sax::Token::TokenType::START_ELEMENT);
	for (auto i : graph.nodeValues) {
		composeNode(out, i.first);
		out.emplace_back("value", sax::Token::TokenType::START_ELEMENT);
		out.emplace_back(ext::to_string(i.second), sax::Token::TokenType::CHARACTER);
		out.emplace_back("value", sax::Token::TokenType::END_ELEMENT);
	}
	out.emplace_back("nodevalues", sax::Token::TokenType::END_ELEMENT);
}

template<typename T>
void GraphToXMLComposer::composeEdgeValues(ext::deque<sax::Token> &out, const T &graph) {
	out.emplace_back("edgevalues", sax::Token::TokenType::START_ELEMENT);
	for (auto &i : graph.edgeValues) {
		composeEdge(out, i.first);
		out.emplace_back("value", sax::Token::TokenType::START_ELEMENT);
		out.emplace_back(ext::to_string(i.second), sax::Token::TokenType::CHARACTER);
		out.emplace_back("value", sax::Token::TokenType::END_ELEMENT);
	}
	out.emplace_back("edgevalues", sax::Token::TokenType::END_ELEMENT);
}

} // namespace graph

#endif // GRAPH_TO_XML_COMPOSER_H_
