/*
 * bitset.hpp
 *
 * Created on: May 04, 2017
 * Author: Jan Travnicek
 */

#ifndef __BITSET_HPP_
#define __BITSET_HPP_

#include <bitset>
#include <ostream>
#include <string>
#include <sstream>

#include "compare.hpp"

namespace ext {

template < std::size_t N >
class bitset : public std::bitset < N > {
public:
#ifdef __clang__
	using std::bitset < N >::bitset;
	using std::bitset < N >::operator =;
#else
	bitset ( ) noexcept : std::bitset < N > ( ) {
	}

	bitset ( const bitset & other ) noexcept : std::bitset < N > ( other ) {
	}

	bitset ( bitset && other ) noexcept : std::bitset < N > ( std::move ( other ) ) {
	}

	using std::bitset < N >::bitset;

	bitset & operator = ( bitset && other ) noexcept {
		static_cast < std::bitset < N > & > ( * this ) = std::move ( other );
		return * this;
	}

	bitset & operator = ( const bitset & other ) noexcept {
		static_cast < std::bitset < N > & > ( * this ) = other;
		return * this;
	}

	using std::bitset < N >::operator =;
#endif
};

template < size_t N >
std::ostream & operator << ( std::ostream & out, const ext::bitset < N > & bitset ) {
	out << "[";

	for ( size_t i = 0; i < N; ++i ) {
		if ( i != 0 ) out << ", ";
		out << bitset [ i ];
	}

	out << "]";
	return out;
}

template < size_t N >
struct compare < ext::bitset < N > > {
	int operator ( ) ( const ext::bitset < N > & first, const ext::bitset < N > & second ) const {
		for ( size_t i = 0; i < N; ++i ) {
			int res = first [ i ] != second [ i ];
			if ( res != 0 )
				return res;
		}
		return 0;
	}
};

template < size_t N >
std::string to_string ( const ext::bitset < N > & value ) {
	std::stringstream ss;
	ss << value;
	return ss.str ( );
}

} /* namespace ext */

#endif /* __BITSET_HPP_ */
