/*
 * GrammarFromStringParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef GRAMMAR_FROM_STRING_PARSER_H_
#define GRAMMAR_FROM_STRING_PARSER_H_

#include "GrammarFromStringLexer.h"
#include "grammar/GrammarFeatures.h"
#include "alphabet/Symbol.h"
#include <set>
#include <vector>
#include <map>
#include <tuple>

namespace grammar {

class GrammarFromStringParser {
	static ext::set<DefaultSymbolType> parseSet(std::istream& input);

	static ext::map<DefaultSymbolType, ext::set<ext::vector<DefaultSymbolType>>> parseCFLikeRules(std::istream& input);

	template<class T>
	static T parseCFLikeGrammar(std::istream& input);
	static ext::map<ext::vector<DefaultSymbolType>, ext::set<ext::vector<DefaultSymbolType>>> parseCSLikeRules(std::istream& input);

	template<class T>
	static T parseCSLikeGrammar(std::istream& input);
	static ext::map<ext::tuple<ext::vector<DefaultSymbolType>, DefaultSymbolType, ext::vector<DefaultSymbolType>>, ext::set<ext::vector<DefaultSymbolType>>> parsePreservingCSLikeRules(std::istream& input);

	template<class T>
	static T parsePreservingCSLikeGrammar(std::istream& input);

public:
	static Grammar parseGrammar(std::istream& input);
	static Grammar parseGrammar(std::istream& input, const ext::set<FEATURES>& features);

	static RightRG < > parseRightRG(std::istream& input);
	static LeftRG < > parseLeftRG(std::istream& input);
	static RightLG < > parseRightLG(std::istream& input);
	static LeftLG < > parseLeftLG(std::istream& input);

	static LG < > parseLG(std::istream& input);
	static CFG < > parseCFG(std::istream& input);
	static EpsilonFreeCFG < > parseEpsilonFreeCFG(std::istream& input);
	static GNF < > parseGNF(std::istream& input);
	static CNF < > parseCNF(std::istream& input);

	static NonContractingGrammar < > parseNonContractingGrammar(std::istream& input);
	static CSG < > parseCSG(std::istream& input);
	static ContextPreservingUnrestrictedGrammar < > parseContextPreservingUnrestrictedGrammar(std::istream& input);
	static UnrestrictedGrammar < > parseUnrestrictedGrammar(std::istream& input);
};

} /* namespace grammar */

#endif /* GRAMMAR_FROM_STRING_PARSER_H_ */
