/*
 * StringToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include <object/ObjectToStringComposer.h>

#include "StringToStringComposer.h"
#include "string/CyclicString.h"
#include "string/LinearString.h"
#include "string/Epsilon.h"

#include <registration/AlgoRegistration.hpp>

namespace string {

template < class SymbolType >
void StringToStringComposer::compose ( std::ostream & out, const CyclicString < SymbolType > & string ) {
	out << "<";

	bool first = true;
	for ( const auto & symbol : string.getContent ( ) ) {
		if ( first )
			first = false;
		else
			out << " ";
		alib::ObjectToStringComposer::compose ( out, symbol );
	}

	out << ">";
}

auto StringToStringComposerCyclicString = registration::OverloadRegister < StringToStringComposer, void, CyclicString < > > ( StringToStringComposer::compose );
auto ObjectToStringComposerCyclicString = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, CyclicString < > >( StringToStringComposer::compose);

template < class SymbolType >
void StringToStringComposer::compose ( std::ostream & out, const LinearString < SymbolType > & string ) {
	out << "\"";

	bool first = true;
	for ( const auto & symbol : string.getContent ( ) ) {
		if ( first )
			first = false;
		else
			out << " ";
		alib::ObjectToStringComposer::compose ( out, symbol );
	}

	out << "\"";
}

auto StringToStringComposerLinearString = registration::OverloadRegister < StringToStringComposer, void, LinearString < > > ( StringToStringComposer::compose );
auto ObjectToStringComposerLinearString = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, LinearString < > >( StringToStringComposer::compose);

template < class SymbolType >
void StringToStringComposer::compose ( std::ostream & out, const Epsilon < SymbolType > & ) {
	out << "#E";
}

auto StringToStringComposerEpsilon = registration::OverloadRegister < StringToStringComposer, void, Epsilon < > > ( StringToStringComposer::compose );
auto ObjectToStringComposerEpsilon = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, Epsilon < > >( StringToStringComposer::compose);

void StringToStringComposer::compose ( std::ostream & out, const String & string ) {
	dispatch ( out, string.getData ( ) );
}

auto ObjectToStringComposerString = registration::OverloadRegister < alib::WrapperBaseDispatcher, void, String>(StringToStringComposer::compose);

} /* namespace string */
