/*
 * GraphToStringComposer.cpp
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 *  Modified by: Jan Broz
 */

#include "GraphToStringComposer.h"
#include "graph/GraphClasses.h"
#include <StringApi.hpp>
#include <label/LabelStringApi.hpp>

#include <registration/AlgoRegistration.hpp>

namespace graph {

void GraphToStringComposer::compose(std::ostream &out, const Graph &graph)
{
	out << "(";
	dispatch(out, graph.getData());
	out << ")";
}

void GraphToStringComposer::compose(std::ostream& out, const DirectedGraph &graph)
{
	out << graph.className();
	//composeRepresentation(out, graph.getRepresentation());
	out << ":";
	composeNodes(out, graph.getNodes());
	out << ":";
	composeDirectedEdges(out, graph.getEdges());
	out << ":";
	composeNodeValues(out, graph);
	out << ":";
	composeEdgeValues(out, graph);
}

//auto GraphToStringComposerDirectedGraph = registration::OverloadRegister < GraphToStringComposer, void, DirectedGraph>(GraphToStringComposer::compose);

void GraphToStringComposer::compose(std::ostream& out, const UndirectedGraph &graph)
{
	out << graph.className();
	//composeRepresentation(out, graph.getRepresentation());
	out << ":";
	composeNodes(out, graph.getNodes());
	out << ":";
	composeUndirectedEdges(out, graph.getEdges());
	out << ":";
	composeNodeValues(out, graph);
	out << ":";
	composeEdgeValues(out, graph);
}

//auto> GraphToStringComposerUndirectedGraph = registration::OverloadRegister < GraphToStringComposer, void, UndirectedGraph>(GraphToStringComposer::compose);


// !! add graph types/representations
void composeALDG(std::ostream & out, const AdjacencyListDirectedGraph & graph) {
	GraphToStringComposer::compose(out, graph);
}

auto GraphToStringComposerALDG = registration::OverloadRegister < GraphToStringComposer, void, AdjacencyListDirectedGraph>(composeALDG);

void composeAMDG(std::ostream & out, const AdjacencyMatrixDirectedGraph & graph) {
	GraphToStringComposer::compose(out, graph);
}

auto GraphToStringComposerAMDG = registration::OverloadRegister < GraphToStringComposer, void, AdjacencyMatrixDirectedGraph>(composeAMDG);

void composeALUG(std::ostream & out, const AdjacencyListUndirectedGraph & graph) {
	GraphToStringComposer::compose(out, graph);
}

auto GraphToStringComposerALUG = registration::OverloadRegister < GraphToStringComposer, void, AdjacencyListUndirectedGraph>(composeALUG);

void composeAMUG(std::ostream & out, const AdjacencyMatrixUndirectedGraph & graph) {
	GraphToStringComposer::compose(out, graph);
}

auto GraphToStringComposerAMUG = registration::OverloadRegister < GraphToStringComposer, void, AdjacencyMatrixUndirectedGraph>(composeAMUG);



void GraphToStringComposer::composeNode(std::ostream& out, const Node &node)
{
	alib::stringApi<label::Label>::compose(out, node.getName());
}

void GraphToStringComposer::composeEdge(std::ostream& out, const DirectedEdge &edge)
{
	composeNode(out, edge.getFromNode());
	alib::stringApi<label::Label>::compose(out, edge.getName());
	composeNode(out, edge.getToNode());
}

void GraphToStringComposer::composeEdge(std::ostream& out, const UndirectedEdge &edge)
{
	composeNode(out, edge.getFirstNode());
	alib::stringApi<label::Label>::compose(out, edge.getName());
	composeNode(out, edge.getSecondNode());
}

void GraphToStringComposer::composeRepresentation(std::ostream &out, REPRESENTATION representation)
{
	out << representationToString(representation);
}

void GraphToStringComposer::composeNodes(std::ostream &out, const ext::set<Node> &nodes)
{
	bool first = true;

	for (const Node &node : nodes) {
		if (first) {
			first = false;
		} else {
			out << ",";
		}
		composeNode(out, node);
	}
}

void GraphToStringComposer::composeDirectedEdges(std::ostream &out, const ext::set<DirectedEdge> &edges)
{
	bool first = true;

	for (const DirectedEdge &edge : edges) {
		if (first) {
			first = false;
		} else {
			out << ",";
		}
		composeEdge(out, edge);
	}
}

void GraphToStringComposer::composeUndirectedEdges(std::ostream &out, const ext::set<UndirectedEdge> &edges)
{
	bool first = true;

	for (const UndirectedEdge &edge : edges) {
		if (first) {
			first = false;
		} else {
			out << ",";
		}
		composeEdge(out, edge);
	}
}

template<typename T>
void GraphToStringComposer::composeNodeValues(std::ostream &out, const T &graph)
{
	bool first = true;

	for (auto i : graph.nodeValues) {
		if (first) {
			first = false;
		} else {
			out << ",";
		}
		composeNode(out, i.first);
		out << "=" << i.second;
	}
}

template<typename T>
void GraphToStringComposer::composeEdgeValues(std::ostream &out, const T &graph)
{
	bool first = true;

	for (auto i : graph.edgeValues) {
		if (first) {
			first = false;
		} else {
			out << ",";
		}
		composeEdge(out, i.first);
		out << "=" << i.second;
	}
}

} // namespace graph
