/*
 * LabelToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include <object/ObjectToStringComposer.h>

#include "LabelToStringComposer.h"
#include "label/PrimitiveLabel.h"
#include "label/HexavigesimalLabel.h"
#include "label/ObjectLabel.h"
#include "label/UniqueLabel.h"
#include "label/InitialStateLabel.h"
#include "label/FinalStateLabel.h"
#include "label/FailStateLabel.h"

#include <StringApi.hpp>

#include <registration/AlgoRegistration.hpp>

namespace label {

void LabelToStringComposer::compose(std::ostream& out, const PrimitiveLabel& label) {
	alib::stringApi<primitive::Primitive>::compose(out, label.getData());
}

auto LabelToStringComposerPrimitiveLabel = registration::OverloadRegister < LabelToStringComposer, void, PrimitiveLabel>(LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const HexavigesimalLabel& label) {
	out << (std::string) label;
}

auto LabelToStringComposerHexavigesimalLabel = registration::OverloadRegister < LabelToStringComposer, void, HexavigesimalLabel>(LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const ObjectLabel& label) {
	out << (std::string) label;
}

auto LabelToStringComposerObjectLabel = registration::OverloadRegister < LabelToStringComposer, void, ObjectLabel>(LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const UniqueLabel& label) {
	out << '<';
	dispatch(out, label.getLabel().getData());
	out << ", ";
	alib::stringApi<primitive::Primitive>::compose(out, primitive::Primitive(label.getId()));
	out << '>';
}

auto LabelToStringComposerUniqueLabel = registration::OverloadRegister < LabelToStringComposer, void, UniqueLabel>(LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const FailStateLabel& label) {
	out << (std::string) label;
}

auto LabelToStringComposerFailStateLabel = registration::OverloadRegister < LabelToStringComposer, void, FailStateLabel>(LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const InitialStateLabel& label) {
	out << (std::string) label;
}

auto LabelToStringComposerInitialStateLabel = registration::OverloadRegister < LabelToStringComposer, void, InitialStateLabel>(LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const FinalStateLabel& label) {
	out << (std::string) label;
}

auto LabelToStringComposerFinalStateLabel = registration::OverloadRegister < LabelToStringComposer, void, FinalStateLabel>(LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& output, const Label& label) {
	dispatch(output, label.getData());
}

} /* namespace label */
